import json
import logging
import urllib
import ssl
from flask import Response, Blueprint, request
from barix.web.auth import login_required

bp = Blueprint('groups_API', __name__)
mainUrl = "http://127.0.0.1:8765"

log = logging.getLogger('flask-backend')

def buildResponse(headers, status, payload=None):
    response_headers = {}

    for (key, value) in headers.items():
        if key.lower() in ["content-length", "connection"]:
            continue
        response_headers[key] = value
    if payload is None:
        return Response(status=status, headers=response_headers)
    else:
        return Response(response=payload, status=status, headers=response_headers)



@bp.route('/groups', methods=['GET'])
@login_required
def get_all_groups():
    try:
        # get groups from main backend
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/groups", method="GET")
        res = urllib.request.urlopen(redirection_req, context=context)
        data = res.read()
        enc = res.headers.get_content_charset('utf-8')
        payload = data.decode(enc)
        json_payload = json.loads(payload)

        # transform json into list of groups
        groupsList = []
        for key, value in json_payload.items():
            groupDict = {"id": key}
            for key2, value2 in value.items():
                groupDict[key2] = value2
            groupsList.append(groupDict)

        # send list of groups in response
        response_json = {"groups": groupsList}
        return Response(json.dumps(response_json), status=200, headers={"Content-Type": "application/json"})
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/groups', methods=['POST'])
@login_required
def new_group():
    try:
        # get group data from request body
        payload = request.json
        # send request to main app
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/groups", data=json.dumps(payload).encode('utf-8'), method="POST")
        res = urllib.request.urlopen(redirection_req, context=context)

        return buildResponse(res.headers, res.getcode(), res.read())
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/groups', methods=['PUT'])
@login_required
def edit_group():
    try:
        # get group data from request body
        payload = request.json

        # send request to main app
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/groups", data=json.dumps(payload).encode('utf-8'), method="UPDATE")
        res = urllib.request.urlopen(redirection_req, context=context)

        return buildResponse(res.headers, res.getcode(), res.read())
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/groups', methods=['DELETE'])
@login_required
def delete_group():
    try:
        # get groups list of ids to delete from request body
        groupsList = request.json

        payload = {"group_ids": groupsList}

        # send request to main app
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/groups", data=json.dumps(payload).encode('utf-8'), method="DELETE")
        res = urllib.request.urlopen(redirection_req, context=context)

        return buildResponse(res.headers, res.getcode(), res.read())
    except Exception as e:
        log.error(e)
        return Response(status=500)