import json
import logging
import urllib
import ssl
from flask import Response, send_from_directory, Blueprint, request, abort
from barix.web.uci import getValueOfUci

from auth import token_required

bp = Blueprint('paging_API', __name__)
mainUrl = "http://127.0.0.1:8765"
DOWNLOAD_DIRECTORY = "/mnt/data/prerecorded_messages/"

log = logging.getLogger('flask-backend')

def buildResponse(headers, status, payload=None):
    response_headers = {}

    for (key, value) in headers.items():
        if key.lower() in ["content-length", "connection"]:
            continue
        response_headers[key] = value
    if payload is None:
        return Response(status=status, headers=response_headers)
    else:
        return Response(response=payload, status=status, headers=response_headers)


def getUsers():
    users = []
    with open("/barix/apps/icpgw/users_backup.json") as file:
        users = json.load(file)
    return users


def getAudioFiles():
    audio_files = []
    with open("/barix/apps/icpgw/audio_files_backup.json") as file:
        audio_files = json.load(file)
    return audio_files


@bp.route('/paging/groups', methods=['GET'])
@token_required
def get_group():
    try:
        request_headers = {}
        redirectionURL = mainUrl + "/groups"
        for (key, val) in request.headers:
            if key.lower() in ["content-length", "connection"]:
                continue
            request_headers[key] = val
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        reqs = urllib.request.Request(url=redirectionURL, headers=request_headers, method=request.method)
        res = urllib.request.urlopen(reqs, context=context)
        data = res.read()
        enc = res.headers.get_content_charset('utf-8')
        payload = data.decode(enc)
        response_headers = {}
        for (key, value) in res.headers.items():
            if key.lower() in ["content-length", "connection"]:
                continue
            response_headers[key] = value
        log.info(
            "Sending Response of {} {} --> Headers: {} Status Code: {} Body: {}".format(request.method,
                                                                                        request.full_path,
                                                                                        response_headers, res.status,
                                                                                        payload))
        return buildResponse(response_headers, res.status, payload)
    except urllib.error.HTTPError as e:
        log.error(e)
        return buildResponse(e.headers, e.code)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/paging/users/<token>/name', methods=['GET'])
@token_required
def get_user(token):
    try:
        # token = request.args['token']
        log.info("Received: {} {}".format(request.method, request.full_path))
        payload = None
        statusCode = None

        users = getUsers()
        for key in users:
            if users[key]["secret"] == token:
                payload = json.dumps({"name": users[key]["name"]})
                statusCode = 200
        if payload is None:
            statusCode = 404

        headers = {'Content-type': 'application/json'}

        return Response(response=payload, status=statusCode, headers=headers)
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/paging/sample-rate', methods=['GET'])
@token_required
def get_sample_rate():
    try:
        sample_rate = getValueOfUci('icpgw', 'barp', 'sample_rate')
        headers = {'Content-type': 'application/json'}
        return Response(response=sample_rate, status=200, headers={'Content-type': 'application/json'})
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/paging/device-messages', methods=['GET'])
@token_required
def get_all_device_messages():
    try:
        user_audio_files = {}
        # get the ids of the audio_files assigned to the user
        user_audio_file_ids = []
        token = request.args['token']
        users = getUsers()
        isAdmin = False
        for user_id in users:
            if users[user_id]["secret"] == token:
                if user_id == "0": # admin
                    isAdmin = True
                else:
                    if "audio_files" in users[user_id].keys():
                        user_audio_file_ids = users[user_id]["audio_files"]


        # get audio files information
        if isAdmin or len(user_audio_file_ids) > 0:
            audio_files = getAudioFiles()
            for key in audio_files:
                if isAdmin:
                    user_audio_files[key] = audio_files[key]
                else:
                    if key in user_audio_file_ids:
                        user_audio_files[key] = audio_files[key]

        headers = {'Content-type': 'application/json'}
        return Response(response=json.dumps(user_audio_files), status=200, headers=headers)
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/paging/device-messages/<filename>/file', methods=['GET'])
@token_required
def get_device_message_file(filename):
    try:
        return send_from_directory(DOWNLOAD_DIRECTORY, filename, as_attachment=True)
    except FileNotFoundError as e:
        log.error(e)
        abort(500)


def iterform(multidict):
    for key in multidict.keys():
        for value in multidict.getlist(key):
            yield (key.encode("utf8"), value.encode("utf8"))


@bp.route('/paging/device-messages/play-file', methods=['POST'])
@token_required
def play_device_message_on_device():
    try:
        payload = json.loads(request.data)
        payload["type"] = 3
        method = 'POST'
        return handle_redirect_request(mainUrl + "/playfile", request.headers, json.dumps(payload).encode('utf-8'), method)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/paging/device-messages/stop-file', methods=['POST'])
@token_required
def stop_device_message_on_device():
    try:
        payload = json.loads(request.data)
        payload["type"] = 8
        method = 'POST'
        return handle_redirect_request(mainUrl + "/playfile", request.headers, json.dumps(payload).encode('utf-8'), method)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/paging/device-messages/play-status', methods=['POST'])
@token_required
def get_status_device_message_on_device():
    try:
        payload = json.loads(request.data)
        payload["type"] = 128
        method = 'PUT'
        return handle_redirect_request(mainUrl + "/playfile", request.headers, json.dumps(payload).encode('utf-8'), method)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/paging/start', methods=['POST'])
@token_required
def startPaging():
    try:
        payload = json.loads(request.data)
        payload["type"] = 3
        method = 'POST'
        return handle_redirect_request(mainUrl + "/paging", request.headers, json.dumps(payload).encode('utf-8'), method)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/paging/stop', methods=['POST'])
@token_required
def stopPaging():
    try:
        payload = json.loads(request.data)
        payload["type"] = 8
        method = 'POST'
        return handle_redirect_request(mainUrl + "/paging", request.headers, json.dumps(payload).encode('utf-8'), method)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/paging/page', methods=['POST'])
@token_required
def doPaging():
    try:
        method = 'PUT'
        return handle_redirect_request(mainUrl + "/paging", request.headers, request.data, method)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/recording/start-paging', methods=['POST'])
@token_required
def startRecording():
    try:
        # log.info("Received: {} {} {}".format(request.method, request.full_path, request.data))
        payload = json.loads(request.data)
        payload["type"] = 3
        method = 'POST'
        return handle_redirect_request(mainUrl + "/recording", request.headers, json.dumps(payload).encode('utf-8'), method)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/recording/stop-paging', methods=['POST'])
@token_required
def stopRecording():
    try:
        payload = json.loads(request.data)
        payload["type"] = 8
        method = 'POST'
        return handle_redirect_request(mainUrl + "/recording", request.headers, json.dumps(payload).encode('utf-8'), method)
    except Exception as e:
        log.error(e)
        abort(500)


@bp.route('/recording/page', methods=['POST'])
@token_required
def pageRecorded():
    try:
        method = 'PUT'
        return handle_redirect_request(mainUrl + "/recording", request.headers, request.data, method)
    except Exception as e:
        log.error(e)
        abort(500)


def handle_redirect_request(redirection_url, req_headers, data, method):
    try:
        redirection_request_headers = {}
        for (key, val) in req_headers:
            if key.lower() in ["content-length", "connection"]:
                continue
            redirection_request_headers[key] = val
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=redirection_url, data=data,
                                                 headers=redirection_request_headers,
                                                 method=method)
        res = urllib.request.urlopen(redirection_req, context=context)
        data = res.read()
        enc = res.headers.get_content_charset('utf-8')
        payload = data.decode(enc)
        response_headers = {}
        for (key, value) in res.headers.items():
            if key.lower() in ["content-length", "connection"]:
                continue
            response_headers[key] = value
        # log.info("Sending Response of {} {} --> Headers: {} Status Code: {} Body: {}".format(request.method, request.full_path, response_headers, res.status, payload))
        return buildResponse(response_headers, res.status, payload)
    except urllib.error.HTTPError as e:
        log.error(e)
        return buildResponse(e.headers, e.code, e.reason)
    except Exception as e:
        log.error(e)
        abort(500)