import logging

from barix.web.certificates import generateSSLCerts
from barix.web.auth import generateNewSessionKey
from barix.web.uci import getValueOfUci

log = logging.getLogger('flask-backend')

def createListOfServicesToRestart(ucis_list):

    # perform aditional actions before restarting services
    checkIfWebProtocolChanged(ucis_list)

    #create list of services
    servicesToRestart = []
    for uci in ucis_list:
        try:
            config = uci[0]
            section = uci[1]
            option = uci[2]
            if config == "application":
                if section == "audio" and option == "volume":
                    servicesToRestart.append("{}.{}.{}".format(config, section, option))
            elif config == "device":
                #log.debug("Changes on device uci config do not require do restart any service")
                pass
            elif config == "dropbear":
                if section == "RunCtl" and option == "enable":
                    servicesToRestart.append("dropbear")
            elif config == "flexa_agent":
                pass
            elif config == "flexa_app":
                #    listOfServicesToRestart.append("S92flexa_app")
                pass
            elif config == "httpd":
                section = uci[1]
                option = uci[2]
                if section == "webserver" and (option == "protocol" or option == "session_timeout"):
                    servicesToRestart.append("backend_flask") # restart backend from new_webui
            elif config == "icpgw":
                servicesToRestart.append("icpgw_main_app")
            elif config == "network":
                servicesToRestart.append('reboot')
            elif config == "ntp":
                servicesToRestart.append("S20ntpd")
            elif config == "security":
                # log.debug("Changes on security uci config do not require do restart any service")
                pass
            elif config == "sonic_ip":
                # log.debug("Changes on sonic_ip uci config do not require do restart any service")
                #listOfServices.append("S13sonic-ip")
                pass
            elif config == "syslogd":
                pass
            elif config == "timezone":
                servicesToRestart.append('S20timezone-handler')
            else:
                log.error("Unrecognized UCI config: {}".format(config))
        except Exception as e:
            log.error(e)
            raise e

    return servicesToRestart

def checkIfWebProtocolChanged(ucisChanged):
    try:
        # check if web protocol changed to HTTPS
        for uciTuple in ucisChanged:
            if ('httpd', 'webserver', 'protocol') == uciTuple:
                generateNewSessionKey()
                if getValueOfUci('httpd', 'webserver', 'protocol') == 'https':
                    generateSSLCerts()
                break
    except Exception as e:
        raise e
