import json
import logging
import urllib
import ssl
from flask import Response, send_from_directory, Blueprint, request, abort
from barix.web.auth import login_required

bp = Blueprint('users_API', __name__)
mainUrl = "http://127.0.0.1:8765"

log = logging.getLogger('flask-backend')

def buildResponse(headers, status, payload=None):
    response_headers = {}

    for (key, value) in headers.items():
        if key.lower() in ["content-length", "connection"]:
            continue
        response_headers[key] = value
    if payload is None:
        return Response(status=status, headers=response_headers)
    else:
        return Response(response=payload, status=status, headers=response_headers)


@bp.route('/users', methods=['GET'])
@login_required
def get_all_users():
    try:
        # get users from main backend
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/users", method="GET")
        res = urllib.request.urlopen(redirection_req, context=context)
        data = res.read()
        enc = res.headers.get_content_charset('utf-8')
        payload = data.decode(enc)
        json_payload = json.loads(payload)

        # add audio_files key to user admin dict
        json_payload["0"]["audio_files"] = []

        # transform json into list of users
        usersList = []
        for key, value in json_payload.items():
            userDict = {"id": key}
            for key2, value2 in value.items():
                userDict[key2] = value2
            usersList.append(userDict)

        # send list of users in response
        response_json = {"users": usersList}
        return Response(json.dumps(response_json), status=200, headers={"Content-Type": "application/json"})
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/users', methods=['POST'])
@login_required
def new_user():
    try:
        # get user data from request body
        payload = request.json

        # send request to main app
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/users", data=json.dumps(payload).encode('utf-8'), method="POST")
        res = urllib.request.urlopen(redirection_req, context=context)

        return buildResponse(res.headers, res.getcode(), res.read())
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/users', methods=['PUT'])
@login_required
def edit_user():
    try:
        # get user data from request body
        payload = request.json

        # send request to main app
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/users", data=json.dumps(payload).encode('utf-8'), method="UPDATE")
        res = urllib.request.urlopen(redirection_req, context=context)

        return buildResponse(res.headers, res.getcode(), res.read())
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/users', methods=['DELETE'])
@login_required
def delete_users():
    try:
        # get users list of ids to delete from request body
        usersList = request.json
        payload = {"id": usersList}

        # send request to main app
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/users", data=json.dumps(payload).encode('utf-8'), method="DELETE")
        res = urllib.request.urlopen(redirection_req, context=context)

        return buildResponse(res.headers, res.getcode(), res.read())
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/users/<id>/token', methods=['POST'])
@login_required
def renew_user_token(id):
    try:
        # send request to main app
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        redirection_req = urllib.request.Request(url=mainUrl + "/users/"+id+"/token", method="POST")
        res = urllib.request.urlopen(redirection_req, context=context)

        return buildResponse(res.headers, res.getcode(), res.read())
    except Exception as e:
        log.error(e)
        return Response(status=500)