import os
import time
from os import path
from threading import Thread
from typing import Union

import alsaaudio
import shared_variables
import constants
from utils import get_logger
import audioop

# configure logger
logger = get_logger('MessageRecordService')


class MessageRecordService:
    """Class to control the audio(capture) aspect for recording a custom message
    methods:
        run  : Start the service.
        exit : Stops the service.
    For usage see paging_server's class definition.
    """

    #MAX_NEW_FILES = set([str(i) for i in range(1, 5)])
    MAX_NEW_FILES = set([str(i) for i in range(1, 2)])

    def __init__(self):
        self.__exit_flag = False
        if shared_variables.config["audio_format"] == "alaw":
            self.__audio_codec = alsaaudio.PCM_FORMAT_A_LAW
        elif shared_variables.config["audio_format"] == "ulaw":
            self.__audio_codec = alsaaudio.PCM_FORMAT_MU_LAW
        else:
            raise Exception(
                "Audio codec not recognized by Message Record service, valid option 'ulaw' and 'alaw'."
            )
        self.__message_record_thread = None
        self.__frequency = shared_variables.config["sample_rate"]
        # self.__play_device = shared_variables.config["output_device"]
        # self.__record_device = shared_variables.config["input_device"]
        # self.__period_size = shared_variables.config["sample_rate"]
        # self.__num_channel = shared_variables.config["num_channels"]

        # self.__message_path = path.join("/mnt", "data", "recorded_messages")
        self.__message_path = constants.PROCESSED_FILES_PATH
        # logger.info(f"Path to store files: {self.__message_path}")
        # Initialize directory structure
        # pathlib.Path(self.__message_path).mkdir(parents=True, exist_ok=True)
        self.occupied_new_files = set([])
        self.current_file = self.new_file_name

        self.is_running = False

    @property
    def new_id(self) -> Union[None, str]:
        """
            Gets an unoccupied new file id.
        """
        available_ids = MessageRecordService.MAX_NEW_FILES - self.occupied_new_files
        # logger.info(f"Available IDs: {available_ids}")

        if len(available_ids) == 0:
            logger.warning("Starting to overwrite files")
            self.occupied_new_files = set([])
            # self.current_file = self.new_file_name
            available_ids = MessageRecordService.MAX_NEW_FILES - self.occupied_new_files

        return min(available_ids)

    @property
    def new_file_name(self):
        new_id = self.new_id
        self.occupied_new_files.add(new_id)
        file_name = "newMessage_" + new_id + ".rec"
        full_path = path.join(self.__message_path, file_name)
        # logger.info(f"Full path for new file: {full_path}")
        open(full_path, mode="w").close()
        if os.path.getsize(full_path) != 0:
            open(full_path, mode="w").close()  # Overwrite existing file
        return full_path

    def run(self, message, new_file):
        if self.__message_record_thread is None:
            self.__message_record_thread = Thread(target=self._record_audio, args=(message, new_file))
            self.__message_record_thread.daemon = True
            self.__message_record_thread.start()

    def exit(self):
        self.current_file = self.new_file_name
        self.__exit_flag = True
        # logger.info(f"Inside the exit method. Flag status: {self.__exit_flag}")
        if self.__message_record_thread is not None:
            self.__message_record_thread.join()
            self.__message_record_thread = None
        self.__exit_flag = False

    # TODO: Not working
    def _record_audio(self, audio_input_dev, new_file=1):
        logger.info(f"Start recording message.")
        self.is_running = True

        logger.info(f"Input data type: {type(audio_input_dev)}")
        # Open a new file, otherwise append to the previous one
        if new_file:
            self.current_file = self.new_file_name
            logger.info(f"Created new file: {self.current_file}")
        else:
            logger.info(f"Appending to existing file: {self.current_file}")
        data_block_size = 1920
        i = 0
        with open(self.current_file, "ab") as message_file:
            # logger.info(f"Message to be written: {audio_input_dev}")
            while not self.__exit_flag:
                if len(audio_input_dev) >= data_block_size:
                    message_file.write(
                        audioop.lin2ulaw(audio_input_dev, 2))  #[i * data_block_size: (i + 1) * data_block_size]
                    i = i + 1
                time.sleep(
                    0.001
                )  # NOTE: Prevents excessive CPU usage in case talk is False.
        logger.info(f"File closed: {message_file.closed}")
        self.is_running = False
        logger.info(f"Stop recording message")
