import alsaaudio
from paging_master import PagingMaster
from utils import uci_to_config_dict, read_json, check_uci_pregong, check_wav_file_format
from groups import Groups
from users import Users
from audio_files import AudioFiles  # Anton
from audio_files_processor import AudioFilesProcessor  # Anton
from record_service import MessageRecordService  # Anton
from backup_settings_wd import BackupSettingsWD  # Anton
import constants  # Anton
from utils import get_logger

import os

#from rtp_processor import RtpProcessor 
from paging_controller import PagingController
from modbus_server import Modbus 

from watchDog import WatchDog

import time

logger = get_logger("SharedVariables")




def initialize():
    global config
    global pmaster
    global last_message_type
    global active_browser
    global TIMEOUT
    global groups
    global users
    
    
    #global rtp_processor
    global paging_controller
    global modbus_server
    global loop_file_infinitly
    global modbus_message
    global modbus_running

    global pregong_playing
    global pregong_message 
    global pregong_function
    global webRequest_playing_file

    global watchDog_live_paging

    global audio_files  # Anton
    global audio_processor  # Anton
    global playfile_state  # Anton
    global delete_processed_files  # Anton
    global downloading_recorded_message  # Anton
    global paging_message_q
    global paging_message_q_buff_msg
    global paging_state
    global record_service
    global backup_files
    global local_alsa_player
    global last_client
    global user_token_length
    global active_user
    global active_user_uuid
    global first_fast_packets

    # Thread handles
    global local_playback_queue_process
    global local_playback_process
    
    global paging_message_q_thread
    global send_page_group_thread
    global playfile_group_thread
    global playfile_playing_thread

    # Flags
    global exit_now
    global end_operation
    global kill_playfile_thread  # Anton
    global kill_local_playback_thread  # Anton

    global local_playback_msg_queue
    #global last_click_time


    playfile_group_thread = None
    playfile_playing_thread = None

    #last_click_time = -1
    exit_now = False
    
    local_playback_queue_process = None
    local_playback_msg_queue = None
    local_playback_process = None

    end_operation = True  # Only used to stop local playback thread

    send_page_group_thread = None

    active_user = None
    active_user_uuid = None

    last_client = None
    groups = Groups()
    users = Users()

    paging_state = constants.IDLE
    paging_message_q = None
    paging_message_q_thread = None
    paging_message_q_buff_msg = 0
    first_fast_packets = 0
    
    
    playfile_state = constants.IDLE  # Anton


    delete_processed_files = constants.YES  # Anton
    kill_playfile_thread = constants.NO  # Anton
    downloading_recorded_message = constants.YES  # Anton
    kill_local_playback_thread = constants.NO  # Anton

    audio_processor = AudioFilesProcessor()  # Anton
    audio_files = AudioFiles()  # Anton

    user_token_length = len(users.new_secret)
    config = uci_to_config_dict()


    loop_file_infinitly = False
    modbus_running = False
    modbus_message = {}
    pregong_playing = False
    pregong_message = ""
    pregong_function = None
    modbus_server = None
    webRequest_playing_file = False

    #rtp_processor = RtpProcessor()
    paging_controller = PagingController()

    if config["modbus_enabled"]:
        modbus_server = Modbus(callback_play_audio, config['modbus_port'])

    record_service = MessageRecordService()  # Anton

    watchDog_live_paging = WatchDog()


    TIMEOUT = config["browser_timeout"]  # seconds
    active_browser = {"time": None, "id": ""}
    pmaster = PagingMaster(config["paging_master_id"])
    last_message_type = 8

    backup_files = BackupSettingsWD(["*_backup.json"])  # Anton

    local_alsa_player = alsaaudio.PCM(
        alsaaudio.PCM_PLAYBACK, alsaaudio.PCM_NORMAL, config["audio_device"]["playback"]  # plug:master1
    )
    local_alsa_player.setchannels(1)
    local_alsa_player.setperiodsize(32)  # 2972
    local_alsa_player.setrate(config["sample_rate"])
    local_alsa_player.setformat(alsaaudio.PCM_FORMAT_S16_LE)


def get_new_alsa_player(codec):
    if codec == "MULAW":
        player_format = alsaaudio.PCM_FORMAT_MU_LAW
    elif codec == "ALAW":
        player_format = alsaaudio.PCM_FORMAT_A_LAW
    else:
        player_format = alsaaudio.PCM_FORMAT_S16_LE

    alsa_player = alsaaudio.PCM(
        alsaaudio.PCM_PLAYBACK, alsaaudio.PCM_NORMAL, config["audio_device"]["playback"]  # plug:master1
    )
    alsa_player.setchannels(1)
    alsa_player.setperiodsize(32)  # 2972
    alsa_player.setrate(config["sample_rate"])
    alsa_player.setformat(player_format)
    return alsa_player


def write_to_local_playback(data, live_paging=False, alsa_player=None, codec="PCM"):
    # logger.info(f"kill_local_playback_thread: {kill_local_playback_thread}")

    #logger.debug(f"write_to_local_playback")

    if alsa_player is None:
        # player = get_new_alsa_player(codec=codec)
        player = local_alsa_player
    else:
        player = alsa_player

    if live_paging:
        logger.info("live paging")
        try:
            player.write(data)
        except Exception as e:
            logger.error(f"player.write(data): {e}")
    else:
        global kill_local_playback_thread
        global loop_file_infinitly
        global modbus_message
        global modbus_running
        global playfile_state
        global paging_state
        global config
        global pregong_playing

        global webRequest_playing_file

        logger.info("Starting local playback")
        kill_local_playback_thread = constants.NO

        data_block_size = 128
        leftover = len(data) % data_block_size
        count = 0
        try:
            for i in range(len(data) // data_block_size):
                if not kill_local_playback_thread:
                    # logger.info(f"Adding packet: {i}, kill_local_playback_thread: {kill_local_playback_thread}")
                    player.write(data[i * data_block_size: (i + 1) * data_block_size])
                    count = count + data_block_size

                    #if i%10 == 0:
                    #    logger.debug(f"write_to_local_playback - {i} // {len(data) // data_block_size}")

                else:
                    break

            if leftover and not kill_local_playback_thread:
                logger.info("Sending leftover")
                player.write(data[len(data) - leftover:])
                count = count + leftover
            logger.info(f"Played locally: {count / 2}, out of {len(data) / 2} bytes")
            logger.info("Finished local playback")


            if playfile_state or paging_state:
                logger.debug(f"RESOURCES BUSY: playfile_state {playfile_state} - paging_state {paging_state}")

            while playfile_state or paging_state:
                time.sleep(0.001)


            if pregong_playing:
                pass
                
                #start_playing_file(pre)
                logger.debug("finished pregong - play normal file")
                if pregong_function != None:
                    logger.debug("call pregong_function")
                    pregong_function(pregong_message)

                pregong_playing = False
                return

            if webRequest_playing_file == True:

                logger.debug("webRequest_playing_file finished!!!")

                webRequest_playing_file = False
                #return
            #logger.debug(f"############################## pregong_playing - {pregong_playing} ############################## ")


            if config["modbus_enabled"] == True and modbus_running == True:
                logger.debug( "write_to_local_playback" )
                data = modbus_server.block.getValues(2,3)

                #file_name = get_file_name_from_modbus_idx(data[0])
                #if file_name == None:
                #    logger.error("Modbus idx not found")
                #    return
                #else:
                #    logger.debug(f"found file {file_name}")
#
                #modbus_message = {
                #    'file_to_open': file_name,
                #    'groups':[data[1]],
                #    'type': 3 # start
                #    }

                if data[0] == 0:
                    modbus_running = False
                    return

                
                if loop_file_infinitly == True:
                    start_playing_file(modbus_message.copy())
                    return
                elif (modbus_message['n_reps'] > 0):
                    modbus_message['n_reps'] = modbus_message['n_reps'] - 1
                    logger.debug(f"modbus_message - {modbus_message}")
                    data = modbus_server.block.setValues_no_callback(4, modbus_message['n_reps'])
                    start_playing_file(modbus_message.copy())
                    return
                else:
                    data = modbus_server.block.setValues_no_callback(2, [0,0,0])
                    #clear modbus flags to allow regular paging
                    modbus_running = False
            

            
            message={
                'groups': [],
            }
            stop_playing_file(message)

        except Exception as e:
            logger.error(f"player.write(data): {e}")







def kill_alsa_player():
    global kill_local_playback_thread
    kill_local_playback_thread = constants.YES



def check_pregong():
    file_name = ""
    if not check_uci_pregong():
        logger.info("Pregong disabled")
        return False, file_name

    file_list = os.listdir("/mnt/data/pregong/processed")

    if len(file_list) > 0:
        file_name = "/mnt/data/pregong/processed/" + file_list[0]

    if os.path.isfile(file_name):
        return True, file_name

    return False, file_name

def get_file_name_from_modbus_idx(idx):
    list_audio_files = read_json("audio_files_backup.json")
    for i in list_audio_files:
        if "modbus_idx" in list_audio_files[i]:
            if list_audio_files[i]["modbus_idx"] == idx:
                return constants.PROCESSED_FILES_PATH + check_wav_file_format(list_audio_files[i]["file_name"])
    return None

from paging_functions import start_playing_file, stop_playing_file, stop_paging_message
from paging_functions import local_playback_stop

def callback_play_audio():
    global modbus_server
    global modbus_message
    global modbus_running
    global loop_file_infinitly
    
    global config
    
    global pmaster
    global last_client
    global kill_playfile_thread
    global groups


    global downloading_recorded_message
    global first_fast_packets
    global playfile_state

    global watchDog_live_paging

    global kill_local_playback_thread
    


    logger.info("callback_play_audio")
    if config["modbus_enabled"] == False:
        return


    
    data = modbus_server.block.getValues(2,3)

    
    logger.debug(f"file {data[0]}  group {data[1]} n_iterations {data[2]}")
    print(f"file {data[0]}  group {data[1]} n_iterations {data[2]}")

    # modbus sent command to stop!
    # or no group selected
    #if data[0] == 0 or data[1] == 0:
    #    modbus_message["type"] = 8
    #    stop_playing_file(modbus_message)
    #    modbus_running = False
    #    return
    
    if modbus_running == True:
        logger.warning("Modbus already running!!!")

        if data[0] == 0:
            logger.info("Modbus stop")
            data = modbus_server.block.setValues_no_callback(2, [0,0,0])
            modbus_message["type"] = 8
            kill_playfile_thread = constants.YES
            local_playback_stop()
            stop_playing_file(modbus_message)
            modbus_running = False
        return
    
    
    if data[0] == 0:
        logger.debug("nothing to be done")
        return

    # test to see if erros happen
    # before sending stop, verify if it is running

    #if(last_client != None):
    #    logger.debug("stop!!!")
    #    message = {"groups": last_client}
    #    clients = groups.get_clients(message["groups"])
    #    pmaster.send_end_group_request(clients)
    #    kill_playfile_thread = constants.YES
    #



    file_name = get_file_name_from_modbus_idx(data[0])
    if file_name == None:
        logger.error("Modbus idx not found")
        return
    else:
        logger.debug(f"found file {file_name}")


    # check if group exist
    
    

    #_group = []
    
    if data[1] != 0:
        try:
            if groups.get_clients([data[1]]) == []: 
                logger.warning(f"group id {data[1]} don't exist or is empty")
                _group = ""
            else:
                _group = [data[1]]
                
        except Exception as e:
            logger.error(f"group id {data[1]} don't exist or is empty")
            _group = ""
        
    else:
        _group = ""


    

    modbus_running = True
    
    #stop_playing_file(message)
    #stop_paging_message(message)
    local_playback_stop()
    
    #pmaster.send_end_group_request({"groups":last_client})
    kill_playfile_thread = constants.YES

    if pmaster.state == 130:
        logger.warning("FORCE STOP")
        modbus_message["type"] = 8
        modbus_message["groups"] = ""
        modbus_message["path"] = '/paging'
        stop_playing_file(modbus_message)
        stop_paging_message(modbus_message)
        
    watchDog_live_paging.stop()


    #wait for threads to stop
    time.sleep(1)

    # in the case of live paging
    

    # play audio file
        
    if(data[2] == 0):
        loop_file_infinitly = True
    else:
        loop_file_infinitly = False


    

    modbus_message = {
        #'file_ID': 'default.mp3',
        'file_to_open': file_name,
        'groups':_group,
        'type': 3, # start
        'n_reps' : data[2]
        }



    downloading_recorded_message = constants.NO
    first_fast_packets = 0
    kill_playfile_thread = constants.NO
    playfile_state = constants.BUSY
    kill_local_playback_thread = constants.NO

    pregong_found, pregong_file = check_pregong()

    if pregong_found:
        pregong_message = {
            'file_to_open': pregong_file,
            'groups':_group,
            'type': 3 # start
            }

        start_playing_file(pregong_message.copy())
    else:

        if (modbus_message['n_reps'] > 0):
            modbus_message['n_reps'] = modbus_message['n_reps'] -1
            data = modbus_server.block.setValues_no_callback(4, modbus_message['n_reps'])

        start_playing_file(modbus_message.copy())

