#include <baco/audio/SilenceSource.h>
#include <baco/audio/FrameFormat.h>
#include <baco/audio/AlsaSink.h>
#include <baco/audio/FrameReplicator.h>
#include <baco/audio/StreamerEndpoint.h>

#include <baco/Library.h>
#include <baco/ScheduledExecutor.h>
#include <baco/JsonUtil.h>
#include <baco/ClockMonotonic.h>

#include "InputBlock.h"
#include "AudioLoopCore.h"

using namespace baco;
using namespace std;

int main(void)
{
    string rtpOutUrl = "rtp://239.0.0.1:9000";
    CodecId streamerCodecId = CodecId::S16BE;
    int streamerSampleRate       = 44100;
    int streamerChannels         = 2;

    initializeCoreLib();
    initializeAudioLib();
    initializeLogger("/var/log/audioLoopStreamer.log", LogPriority::PRIO_INFO , 3*1024*1024, 1);

    FrameFormat outFmt{ ChannelLayout::STEREO , SampleFormat::I16, 2 , 44100 };
    auto alsaSink{ make_shared<AlsaSink>("plug:master1",outFmt) };

    auto streamer = make_shared<StreamerEndpoint>(rtpOutUrl , streamerCodecId , streamerSampleRate);

    if( streamerChannels == 2){
        streamer->createOutput(outFmt);
    }
    else {
        streamer->createOutput(outFmt,baco::ChannelId::FRONT_LEFT);
    }

    streamer->enable();
    auto replicator =  make_shared<FrameReplicator>();
    replicator->addFrameSink(alsaSink);
    replicator->addFrameSink(streamer);

    InputBlockConfig inputConfig;
    inputConfig.deviceName = "plug:loopback";
    inputConfig.sampleRate = 44100;
    inputConfig.fifoDelay = 92;
    inputConfig.dispatchSize = 16;

    AudioLoopCorePtr audioLoop { make_unique<AudioLoopCore>(alsaSink , outFmt, 1000, 1.0, replicator) };
    audioLoop->setInputLoop(inputConfig);

    while ( true ) {
        audioLoop->iterate();
    }
}
