#!/bin/bash
#
# The purpose of this scrip is to used to develop stage.
#

echo -e "\n  \033[1;30;47m BARIX \033[00m  Build Angular compile environment${1}\n"

FILE="Dockerfile"
if ! test -f "$FILE"; then
  echo "Run this script on same dir as Dockerfile."
  exit 1
fi

PROJECT_NAME="$(jq -j .name ../package.json)-build-env"
PROJECT_VERSION=$(jq -j .version ../package.json)
HOST_PORT=4200
HOST_PROJECT_DIR=$(pwd)

#--------------------------------------------------
# Colors settings
C_RESET="\e[00m"            # Reset colors
C_HIGHLIGHT="\e[0;37;1m"    # Foreground highlight
C_YELLOW="\e[0;33;1m"       # Foreground yellow
C_GREEN="\e[0;32;1m"        # Foreground green
C_ERROR="\e[0;31;1m"        # Foreground error
#--------------------------------------------------

function testExitStatus {
  # $1 - Exit status
  # $2 - Error message
  if [ $1 -ne 0 ]; then
    echo -ne "\n${C_ERROR}ERROR${C_HIGHLIGHT} - ${2}${C_RESET}\n\n"
    exit -1
  fi
}

function getDockerFile  {
  unameOut="$(uname -s)"
  case "${unameOut}" in
      Darwin*)    dockerfile="Dockerfile.macos";;
      *)          dockerfile="Dockerfile"
  esac
  echo $dockerfile
}

echo -ne "\n${C_GREEN}### ${PROJECT_NAME}:${PROJECT_VERSION} ###${C_RESET}\n\n"

FILE="../package.json"
if ! test -f "$FILE"; then
  echo "Run this script on a project dir. Type \"scripts/$(basename ${0})\" on project dir"
  exit 1
fi

# Stop and remove possible running dependent docker container
docker stop ${PROJECT_NAME} -t 8
docker rm ${PROJECT_NAME}

# Remove possible existing docker image
docker images --filter="reference=barix/${PROJECT_NAME}" --format='{{.Repository}}:{{.Tag}}' | xargs docker rmi -f

# Remove all unused containers, networks, images (both dangling and unreferenced)
#docker system prune --force

# Build docker image
IMAGE_NAME="barix/${PROJECT_NAME}:${PROJECT_VERSION}"
dockerfile=$(getDockerFile)
docker build --build-arg UID=$(id -u) --build-arg GID=$(id -g) --build-arg UNAME=$(echo $USER) --file $dockerfile -t ${IMAGE_NAME} .
testExitStatus $? "Can't build docker image. See errors above."

echo -ne "\n${C_GREEN}Docker image \"${C_HIGHLIGHT}${IMAGE_NAME}${C_GREEN}\" created.${C_RESET}\n\n"
