import { NgModule } from '@angular/core';
import {PreloadAllModules, RouterModule, Routes} from '@angular/router';
import {MainPageComponent} from "./paging/main-page/main-page.component";
import {SettingsPageComponent} from "./paging/settings-page/settings-page.component";
import {PagingMenuComponent} from "./paging/group-menu/paging-menu.component";
import {PagingGroupsComponent} from "./paging/paging-group-page/paging-groups.component";
import {DeviceMessagePageComponent} from "./paging/device-message-page/device-message-page.component";
import {MainComponent} from "./main/main.component";
import {SettingsComponent} from "./settings/settings.component";
import {CanDeactivateDeviceMessageGuard, CanDeactivateGuard} from "./can-deactivate/can-deactivate.guard";
import {SystemStatusComponent} from "./system-status/system-status.component";
import {LogsComponent} from "./logs/logs.component";
import {DefaultsComponent} from "./defaults/defaults.component";
import {UpdateTabComponent} from "./update-tab/update-tab.component";
import {RebootComponent} from "./reboot/reboot.component";
import {LoginComponent} from "./login/login.component";
import {PagingSettingsTabComponent} from "./paging-settings-tab/paging-settings-tab.component";
import {UsersManagementTabComponent} from "./users-management-tab/users-management-tab.component";
import {GroupsTabComponent} from "./groups-tab/groups-tab.component";
import {AudioFilesTabComponent} from "./audio-files-tab/audio-files-tab.component";


const routes: Routes = [
    {
        path: '',
        component: MainComponent,
        children: [
            {
                path: '',
                redirectTo: '/settings',
                pathMatch: 'full'
            },
            {
                path: 'settings',
                component: SettingsComponent,
                canDeactivate: [CanDeactivateGuard]
            },
            {
                path: 'paging-settings',
                component: PagingSettingsTabComponent,
                canDeactivate: [CanDeactivateGuard]
            },
            {
                path: 'users',
                component: UsersManagementTabComponent,
            },
            {
                path: 'groups',
                component: GroupsTabComponent,
            },
            {
                path: 'audio-files',
                component: AudioFilesTabComponent,
            },
            {
                path: 'system-status',
                component: SystemStatusComponent
            },
            {
                path: 'logs',
                component: LogsComponent
            },
            {
                path: 'defaults',
                component: DefaultsComponent
            },
            {
                path: 'update',
                component: UpdateTabComponent
            },
            {
                path: 'reboot',
                component: RebootComponent
            },
        ]
    },
    {
        path: "paging",
        children: [
            {
                path: "",
                component: MainPageComponent
            },
            {
                path: "settings",
                component: SettingsPageComponent
            },
            {
                path: "groups",
                component: PagingGroupsComponent
            },
            {
                path: "menu",
                component: PagingMenuComponent
            },
            {
                path: "device-messages",
                component: DeviceMessagePageComponent,
                canDeactivate: [CanDeactivateDeviceMessageGuard]
            },
        ]
    },
    {
        path: 'login',
        component: LoginComponent
    },
    {
        path: '**',
        redirectTo: '/settings',
        pathMatch: 'full'
    }

];


@NgModule({
  imports: [RouterModule.forRoot(routes, {preloadingStrategy: PreloadAllModules})],
  exports: [RouterModule]
})
export class AppRoutingModule {


}
