import {Component, Input, OnInit} from '@angular/core';
import {MatSliderChange} from "@angular/material/slider";
import {
    AbstractControl,
    ControlValueAccessor,
    FormControl,
    NG_VALIDATORS,
    NG_VALUE_ACCESSOR,
    ValidationErrors
} from "@angular/forms";


@Component({
    selector: 'brx-basic-slider',
    templateUrl: './basic-slider.component.html',
    styleUrls: ['./basic-slider.component.scss'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            multi: true,
            useExisting: BasicSliderComponent
        },
        {
            provide: NG_VALIDATORS,
            multi: true,
            useExisting: BasicSliderComponent
        }
    ]
})


export class BasicSliderComponent implements ControlValueAccessor {
    @Input() value: number | null = 0;
    @Input() displayValue: boolean | undefined = true;
    @Input() unit: string | undefined = undefined;
    @Input() tickInterval: string | undefined = undefined;

    onChange = (value) => {
    };

    onTouched = () => {
    };

    touched = false;
    disabled = false;
    isMoving = false;
    display: FormControl = new FormControl();

    constructor() {
    }


    ngOnInit(): void {
        this.display.setValue(this.value);
    }


    ngOnChanges(): void {
        // Update display value if input value change
        // Will not fire any event
        this.display.setValue(this.value);
    }

    onMove(event: MatSliderChange) {
        // console.log("### MOVING", event.value);
        this.display.setValue(event.value);
        this.isMoving = true;
        this.markAsTouched();
    }


    onChangeEvent(event: MatSliderChange) {
        // console.log("### CHANGE", event.value);
        this.value = event.value;
        this.display.setValue(event.value);
        this.isMoving = false;
        this.onChange(event.value);
    }


    registerOnChange(fn: any): void {
        this.onChange = fn;
    }


    registerOnTouched(fn: any): void {
        this.onTouched = fn;
    }


    writeValue(newValue: number): void {
        this.value = newValue;
    }


    markAsTouched() {
        if (!this.touched) {
            this.onTouched();
            this.touched = true;
        }
    }


    setDisabledState(disabled: boolean) {
        this.disabled = disabled;
    }


    validate(control: AbstractControl): ValidationErrors | null {
        const value = control.value;
        if (value <= 0) {
            return {
                mustBePositive: {
                    value
                }
            };
        }
    }


    getDisplayValueClass()  {
        let classes = "display-value";
        if (this.isMoving)  {
            classes += " display-value-disabled";
        }
        return classes;
    }

}
