import {Component, Input, OnInit} from '@angular/core';
import {MatSliderChange} from "@angular/material/slider";
import {
    AbstractControl, ControlContainer,
    ControlValueAccessor,
    FormControl,
    NG_VALIDATORS,
    NG_VALUE_ACCESSOR,
    ValidationErrors
} from "@angular/forms";


@Component({
    selector: 'brx-knob-q',
    templateUrl: './knob-q.component.html',
    styleUrls: ['./knob-q.component.scss'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            multi: true,
            useExisting: KnobQComponent
        },
        {
            provide: NG_VALIDATORS,
            multi: true,
            useExisting: KnobQComponent
        }
    ]
})


export class KnobQComponent implements ControlValueAccessor {
    @Input() value: number | null = 0;
    @Input() unit: string | undefined = undefined;
    @Input() disabled: boolean | undefined = false;
    @Input() formControl: FormControl;

    onChange = (value) => {
    };

    onTouched = () => {
    };

    touched = false;

    knobSize=96;
    knobStrokeWidth=16;

    knobQValueColor="#00A7DB";
    knobRangeColor="#00000044";
    knobQMin=0.1;
    knobQMax=8
    knobQStep=0.1;

    knobFreqValueColor="#46BF46";
    knobLowFreqMin=20;
    knobLowFreqMax=400
    knobLowMidFreqMin=80;
    knobLowMidFreqMax=1000;
    knobHiMidFreqMin=800;
    knobHiMidFreqMax=12500
    knobHiFreqMin=2500;
    knobHiFreqMax=20000
    knobFreqStep=1;

    knobGainValueColor="#FF230D";
    knobGainMin=-24;
    knobGainMax=24
    knobGainStep=1;

    control:    FormControl;

    constructor( private controlContainer: ControlContainer) { }

    ngOnInit() {

        if(this.controlContainer && this.formControl){
            this.control = this.formControl;
        }

    }


    ngOnChanges(): void {
        // Update display value if input value change
        // Will not fire any event
    }

    onMove(event: MatSliderChange) {
        // console.log("### MOVING", event.value);
        this.markAsTouched();
    }


    onChangeEvent(event) {
        // console.log("### CHANGE", event.value);
        this.value = event.value;
        this.onChange(event.value);
    }


    registerOnChange(fn: any): void {
        this.onChange = fn;
    }


    registerOnTouched(fn: any): void {
        this.onTouched = fn;
    }


    writeValue(newValue: number): void {
        this.value = newValue;
    }


    markAsTouched() {
        if (!this.touched) {
            this.onTouched();
            this.touched = true;
        }
    }


    setDisabledState(disabled: boolean) {
        this.disabled = disabled;
    }


    validate(control: AbstractControl): ValidationErrors | null {
        const value = control.value;
        if (value < this.knobQMin) {
            return {
                mustBeGreaterThan: {
                    value: this.knobQMin

                }
            };
        }
        if (value > this.knobQMin) {
            return {
                mustBeLessThan: {
                    value: this.knobQMax
                }
            };
        }
    }

}
