import {Component, Input} from '@angular/core';
import {
    AbstractControl,
    ControlValueAccessor,
    FormControl,
    NG_VALIDATORS,
    NG_VALUE_ACCESSOR,
    ValidationErrors
} from "@angular/forms";


@Component({
    selector: 'brx-mute-button',
    templateUrl: './mute-button.component.html',
    styleUrls: ['./mute-button.component.css'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            multi: true,
            useExisting: MuteButtonComponent
        },
        {
            provide: NG_VALIDATORS,
            multi: true,
            useExisting: MuteButtonComponent
        }
    ]
})


export class MuteButtonComponent implements ControlValueAccessor {
    @Input() muted: boolean = false;

    onChange = (isMuted: boolean) => {
    };

    onTouched = () => {
    };

    touched = false;
    disabled = false;


    registerOnChange(fn: any): void {
        this.onChange = fn;
    }


    registerOnTouched(fn: any): void {
        this.onTouched = fn;
    }


    writeValue(newValue: boolean): void {
        this.muted = newValue === true;
    }


    markAsTouched() {
        if (!this.touched) {
            this.onTouched();
            this.touched = true;
        }
    }


    setDisabledState(disabled: boolean) {
        this.disabled = disabled;
    }


    validate(control: AbstractControl): ValidationErrors | null {
        return {};
    }


    onClick() {
        this.muted = !this.muted;
        this.markAsTouched();
        this.onChange(this.muted);
    }


    getCssClasses() {
        return this.muted ? "is-muted" : "is-not-muted";
    }

}
