import { Component, OnInit } from '@angular/core';
import {DeviceConfigService} from "../services/device-config.service";
import {HttpErrorResponse} from '@angular/common/http';
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {SystemFunctionsService} from "../services/system-functions.service";
import {ToastMessageService} from "../services/toast-message.service";
import {ToastrService} from "ngx-toastr";
import { TopBarService } from '../services/top-bar.service';

@Component({
  selector: 'app-defaults',
  templateUrl: './defaults.component.html',
  styleUrls: ['./defaults.component.css', '../css/general.css']
})
export class DefaultsComponent implements OnInit {

  loadFinished = false;
  defaults_opt = false;
  //complete = false;
  waitingMessage : string;
  rebooting: boolean;

  constructor(private configsService: DeviceConfigService,
              private toastr: ToastrService,
              private toast: ToastMessageService,
              private httpResponseHandler: HTTPResponseHandlerService,
              private systemFuncsService: SystemFunctionsService,
              private topBarService: TopBarService,) { }

  ngOnInit() {
    this.toast.clearToastMessages()
    this.configsService.getUcis({'defaults_opt':'security.defaults.enabled'}).subscribe(data => {
      switch (data.status) {
        case 200:
          //build device form group
          if (data.body["security.defaults.enabled"] === "true") {
            this.defaults_opt = true;
          }
          break;
        case 401:
          this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
          break;
        case 500:
          this.httpResponseHandler.handle500HttpErrorResponse();
          break;
        default:
          console.error("Unexpected return code: ", data.status);
      }
      this.loadFinished = true;
    });
    this.topBarService.updateDeviceLocalTime();
  }

  resetFactoryDefaults(){
    this.configsService.resetFactoryDefaults().subscribe(data => {
      switch (data.status) {
        case 200:
          //this.complete = true;
          this.toast.showSuccessMessage("Settings successfully reverted to factory defaults.");
          this.waitingMessage = "Please wait while the device reboots. The page will be automatically reloaded.";
          this.rebooting = true;
          setTimeout(() => this.reloadWebUI(), 20000);
          break;
        case 401:
          this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
          break;
        default:
          console.error("Unexpected return code: ", data.status);
          this.toast.showErrorMessage("It wasn't possible to reset configurations to Factory Defaults. Please try again or reboot your device if error persists.");
      }
    });
  }

  reloadWebUI(){
    this.systemFuncsService.appIsRunning().subscribe(res => {
        console.log("RES");
        console.log(res);
      if (res.status === 200){
        window.location.href = window.location.protocol + '//' + window.location.hostname;
      } else {
        setTimeout(() => this.reloadWebUI(), 1000);
      }
    }, error => {
        console.log("APP is running ERROR");
        console.log(error);
    });
  }

}
