import { Component, EventEmitter, Input, OnDestroy, OnInit, Output } from '@angular/core';
import {HttpErrorResponse, HttpEventType, HttpResponse} from '@angular/common/http';
import {FirmwareUploadService} from '../services/firmware-upload.service';
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {ToastrService} from 'ngx-toastr';
import {ProgressBarMode} from '@angular/material/progress-bar';
import {SystemFunctionsService} from "../services/system-functions.service";
import {ToastMessageService} from "../services/toast-message.service";
import {FormCanDeactivate} from "../can-deactivate/form-can-deactivate";
import { MatDialog } from '@angular/material/dialog';
import { FwUpdaterStatusModalComponent } from '../fw-updater-status-modal/fw-updater-status-modal.component';


@Component({
    selector: 'app-firmware-updater',
    templateUrl: './firmware-updater.component.html',
    styleUrls: ['./firmware-updater.component.css', '../css/general.css']
})


export class FirmwareUpdaterComponent implements OnInit {

    @Input() fileExtensionsAllowed = [];
    @Input() max_file_size: number;
    @Output() updateStatusEvent = new EventEmitter<string>();

    updateStatus : string;

    uploadSubscription;

    invalid: boolean = true;
    file: File;
    progress: number = 0;
    mode: ProgressBarMode = "indeterminate";
    // showUploadSection: boolean;
    showProgressSection: boolean;
    showFileUploadProgressBar: boolean;
    showProgressBar: boolean;
    progressMessagesStatus = [];
    successColor = '#333333';
    errorColor = 'red';
    error: boolean;

    constructor(private fileUploadService: FirmwareUploadService,
                private httpResponseHandler: HTTPResponseHandlerService,
                private toast: ToastMessageService,
                private systemFuncsService: SystemFunctionsService,
                public dialog: MatDialog) {}


    ngOnInit(): void {
        for (let i = 0; i < this.fileExtensionsAllowed.length; i++) {
            this.fileExtensionsAllowed[i] = this.fileExtensionsAllowed[i].toLowerCase().replace('.', '');
        }
        // this.showUploadSection = true;
        // this.showProgressSection = false;
        // this.showFileUploadProgressBar = false;
        // this.showProgressBar = false;
        // this.error = false;
    }


    onChange(event) {
        this.invalid = true;
        // check if file is valid
        this.file = event.target.files[0];
        if (this.file) {
            // check if file extension is valid
            let fileExtension = this.file.name.split('.').pop();
            if (!this.fileExtensionsAllowed.includes(fileExtension.toLowerCase())) {
                this.invalid = true;
                this.toast.showWarningMessage("Format \"." + fileExtension + "\" not supported!");
            } else {
                this.invalid = false;
            }
            if (!this.invalid) {
                // check if file size is allowed
                if (this.file.size > this.max_file_size) {
                    this.invalid = true;
                    this.toast.showWarningMessage("File is too large!");
                } else {
                    this.invalid = false;
                }
            }
        }
    }


    onUpload() {
        if (!this.invalid) {
            // this.isUpdating = true;
            // this.isUpdatingEvent.emit(this.isUpdating);
            // this.error = false;
            // this.showProgressBar = true;
            this.fileUploadService.getUpdateStatus().subscribe(res => {
                switch (res.status) {
                    case 200:
                        if (res.body === "clear" || res.body === "error") {
                            this.dialog.open(FwUpdaterStatusModalComponent, {
                                minWidth: "70%",
                                height: "55vh",
                                data: this.file,
                                disableClose: true
                            });
                            // this.showProgressBar = false;
                            // this.progressMessagesStatus.push(['Uploading firmware update file...', this.successColor]);
                            // //hide upload file section and show file upload progress bar
                            // // this.showUploadSection = false;
                            // this.showFileUploadProgressBar = true;
                            // this.showProgressSection = true;
                            // this.updateStatus = "uploading";
                            // this.updateStatusEvent.emit(this.updateStatus);
                            // this.uploadSubscription = this.fileUploadService.upload(this.file).subscribe(res => {
                            //     //console.log(res);
                            //     if (res.type === HttpEventType.UploadProgress) {
                            //         this.progress = Math.round(100 * res.loaded / res.total);
                            //         //console.log("Progress:", this.progress);
                            //         if (this.progress === 100) {
                            //             setTimeout(() => {
                            //                 this.showFileUploadProgressBar = false;
                            //                 if (!this.error) this.showProgressBar = true;
                            //             }, 1000);
                            //         }
                            //     } else if (res instanceof HttpResponse) {
                            //         switch (res.status) {
                            //             case 200:
                            //                 this.showProgressBar = false;
                            //                 this.progressMessagesStatus.push(['Firmware update file uploaded with success.', this.successColor]);
                            //                 setTimeout(() => {
                            //                     this.progressMessagesStatus.push(['Installing firmware update...', this.successColor]);
                            //                     this.showProgressBar = true;
                            //                     this.checkUpdateStatus();
                            //                 }, 800);
                            //                 break;
                            //             default:
                            //                 console.error("Unexpected return code: ", res.status);
                            //         }
                            //     } else if (res instanceof HttpErrorResponse) {
                            //         console.error("HTTP Error Response" + res);
                            //         // this.isUpdating = false;
                            //         // this.isUpdatingEvent.emit(this.isUpdating);
                            //         this.updateStatus = "error";
                            //         this.updateStatusEvent.emit(this.updateStatus);
                            //     }
                            // }, err => {
                            //     console.error(err);
                            //     // this.isUpdating = false;
                            //     // this.isUpdatingEvent.emit(this.isUpdating);
                            //     this.updateStatus = "error";
                            //     this.updateStatusEvent.emit(this.updateStatus);
                            //     this.error = true;
                            //     this.progressMessagesStatus.push(['Error uploading file. ', this.errorColor]);
                            //     this.showFileUploadProgressBar = false;
                            //     this.showProgressBar = false;
                            //     if (err.status === 413 || err.statusText === "REQUEST ENTITY TOO LARGE") {
                            //         this.toast.showWarningMessage("File is too large!");
                            //     } else if (err.status === 409 || err.statusText === "CONFLICT") {
                            //         this.toast.showErrorMessage("There is an update already running.");
                            //     } else if (err.error === "No file") {
                            //         this.toast.showErrorMessage("No file provided");
                            //     }
                            // });
                        } else {
                            // this.isUpdating = false;
                            // this.isUpdatingEvent.emit(this.isUpdating);
                            // this.updateStatus = "error";
                            // this.updateStatusEvent.emit(this.updateStatus);
                            // this.error = true;
                            // this.progressMessagesStatus.push(['Error uploading file. ', this.errorColor]);
                            // this.showProgressBar = false;
                            this.toast.showErrorMessage("There is an update already running.");
                        }
                        break;
                    case 401:
                        this.httpResponseHandler.handle401HttpErrorResponse((res as HttpErrorResponse).error);
                        break;
                    case 500:
                        this.httpResponseHandler.handle500HttpErrorResponse();
                        break;
                    default:
                        console.error("Unexpected return code: ", res.status);
                }
            });
        }
    }


    // checkUpdateStatus() {
    //     this.fileUploadService.getUpdateStatus().subscribe(status => {
    //         if (status.status === 200) {
    //             if (status.body === "rebooting") {
    //                 this.showProgressBar = false;
    //                 this.progressMessagesStatus.push(['Update complete!', this.successColor]);
    //                 // this.isUpdating = false;
    //                 this.updateStatus = "rebooting";
    //                 this.updateStatusEvent.emit(this.updateStatus);
    //                 setTimeout(() => { //to allow user to see
    //                     this.progressMessagesStatus.push(['Please wait while the device is rebooting. The page will be automatically reloaded.', this.successColor]);
    //                     setTimeout(() => this.reloadWebUI(), 10000);
    //                 }, 1000);
    //             } else if (status.body === "error") {
    //                 //console.log("Stop checking for status");
    //                 this.showProgressBar = false;
    //                 this.showFileUploadProgressBar = false;
    //                 this.progressMessagesStatus.push(['An error occurred during the installation.', this.errorColor]);
    //                 // this.isUpdating = false;
    //                 this.updateStatus = "error";
    //                 this.updateStatusEvent.emit(this.updateStatus);
    //             } else if (status.body === "finished") {
    //                 //missed reboot
    //                 this.updateStatus = "finished";
    //                 this.updateStatusEvent.emit(this.updateStatus);
    //                 this.reloadWebUI();
    //             } else { //installing
    //                 this.updateStatus = status.body;
    //                 this.updateStatusEvent.emit(this.updateStatus);
    //                 setTimeout(() => {
    //                     this.checkUpdateStatus();
    //                 }, 3000);
    //             }
    //         } else if (status.status === 401) {
    //             // this.isUpdating = false;
    //             this.updateStatus = "error";
    //             this.updateStatusEvent.emit(this.updateStatus);
    //             this.httpResponseHandler.handle401HttpErrorResponse(status.error);
    //         } else {
    //             setTimeout(() => {
    //                 this.checkUpdateStatus();
    //             }, 1000);
    //         }
    //     });
    // }
    //
    //
    // reloadWebUI() {
    //     this.systemFuncsService.appIsRunning().subscribe(res => {
    //         if (res.status === 200) {
    //             window.location.href = window.location.protocol + '//' + window.location.hostname;
    //         } else {
    //             setTimeout(() => this.reloadWebUI(), 1000);
    //         }
    //     });
    // }

}
