import {ChangeDetectorRef, Component, Input, OnInit} from '@angular/core';


@Component({
    selector: 'app-greedy-nav',
    templateUrl: './greedy-nav.component.html',
    styleUrls: ['./greedy-nav.component.css'],
})


export class GreedyNavComponent implements OnInit {

    @Input() menuData;

    mainMenu = [];
    hiddenPrioMenu = [];
    showDropDownPrio = false;
    breakWidths = [];

    constructor( private cdr: ChangeDetectorRef ) {
    }

    ngOnInit(): void {
        this.menuData.forEach(elem => {
            //console.log(elem);
            this.mainMenu.push(elem);
        });
    }

    ngAfterViewInit() {

        // get prio nav elements
        const prioNav = document.getElementById('prio-nav');

        let totalSpace = 0;
        const breakWidths = [];
        let numOfItems = 0;

        // loop over all elements and set sum of widths for each prio nav element
        for (let i = 0; i < prioNav.children.length; i++) {
            totalSpace += prioNav.children[i].clientWidth;
            this.breakWidths.push(totalSpace);
            numOfItems += 1;
        }

        // call calculation method
        this.checkCalculation();
    }

    onResize(event) {
        // call calculation method
        this.checkCalculation();
    }

    checkCalculation() {

        // get current space of parent element of prio-nav
        const availableSpace = document.getElementById('header-toolbar').offsetWidth;

        // get needed space for all elements in prio-nav parent element + last width of visible elements from breakwidth array
        const totalNeededSpace = this.breakWidths[this.mainMenu.length - 1] + 70;

        // if we need more space than we have - hide last element
        if (totalNeededSpace > availableSpace) {

            // push last element from mainMenu to hiddenPrioMenu
            //this.hiddenPrioMenu.push(this.mainMenu[this.mainMenu.length-1]);
            this.hiddenPrioMenu.unshift(this.mainMenu[this.mainMenu.length - 1]);

            // remove that last element from mainMenu
            this.mainMenu = this.mainMenu.filter(item => item !== this.mainMenu[this.mainMenu.length - 1]);

            // show hidden prio menu
            this.showDropDownPrio = true;

            // apply changes
            this.cdr.detectChanges();

            // call this method to recalculate for rest of nav elements
            this.checkCalculation();

        } else {

            // if we have elements in hidden prio menu and the with occupied by current menu items + 1 from hiddenPrioMenu is still under or equal to the availableSpace

            if (this.hiddenPrioMenu.length > 0 && (this.breakWidths[this.mainMenu.length] + 70) <= availableSpace) {

                // push last element from hidden menu to main menu
                this.mainMenu.push(this.hiddenPrioMenu.shift());

                // remove element from hidden menu
                //this.hiddenPrioMenu = this.hiddenPrioMenu.filter(item => item !== this.hiddenPrioMenu[this.hiddenPrioMenu.length-1]);

                // if we have or not elements in hiddenPrioMenu hide it or show it
                this.showDropDownPrio = this.hiddenPrioMenu.length > 0;

                // apply changes
                this.cdr.detectChanges();

                // call this method to recalculate for rest of nav elements
                this.checkCalculation();

            }

            // apply changes
            this.cdr.detectChanges();
        }
    }
}
