import {Component, OnInit} from '@angular/core';
import {FormGroup, FormControl} from '@angular/forms';
import {first} from 'rxjs/operators';
import {AccountService} from '../services/account.service';
import {ActivatedRoute, Router} from '@angular/router';
import {ToastrService} from 'ngx-toastr';
import {HttpErrorResponse} from '@angular/common/http';
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {ToastMessageService} from "../services/toast-message.service";


@Component({
    selector: 'app-login',
    templateUrl: './login.component.html',
    styleUrls: ['./login.component.css']
})


export class LoginComponent implements OnInit {


    form = new FormGroup({});
    submitted = false;
    host = "";
    loginCanceled = false;

    constructor(private accountService: AccountService,
                private router: Router,
                private toastr: ToastrService,
                private httpResponseHandler: HTTPResponseHandlerService) {
    }


    ngOnInit(): void {
        this.toastr.clear();
        this.host = window.location.protocol + "//" + window.location.host;
        //TODO: validators
        this.form.addControl('username', new FormControl<string>(''));
        this.form.addControl('password', new FormControl<string>(''));
    }

    onSubmit() {

        this.submitted = true;

        // stop here if form is invalid
        if (this.form.invalid) {
            return;
        }

        this.accountService.login(this.form.get('username').value, this.form.get('password').value)
            .subscribe(resp => {
                switch (resp.status) {
                    case 200:
                        this.router.navigate(['/settings']);
                        break;

                    case 401:
                        this.httpResponseHandler.handle401HttpErrorResponse((resp as HttpErrorResponse).error);
                        break;

                    case 500: //INTERNAL_SERVER_ERROR
                        this.httpResponseHandler.handle500HttpErrorResponse();
                        break;

                    default:
                        console.error("Unexpected return code: ", resp.status);
                        let error_msg = "An unexpected HTTP Status code was received: " + resp.status + ". Please try again or reboot your device if error persists."
                        this.toastr.error(error_msg, '', {
                            closeButton: true,
                            timeOut: 10000,
                            positionClass: 'absolute-top-right'
                        });
                }
            });
    }


    cancelLogin() {
        this.loginCanceled = true;
    }

}
