import { Component, OnInit } from '@angular/core';
import {SystemInfoService} from "../services/system-info.service";
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {HttpErrorResponse} from '@angular/common/http';
import {ToastrService} from "ngx-toastr";
import {ToastMessageService} from "../services/toast-message.service";
import { TopBarService } from '../services/top-bar.service';

@Component({
  selector: 'app-logs',
  templateUrl: './logs.component.html',
  styleUrls: ['./logs.component.css', '../css/general.css']
})
export class LogsComponent implements OnInit {

  constructor(private getSystemInfo: SystemInfoService,
              private httpResponseHandler: HTTPResponseHandlerService,
              private toast: ToastMessageService,
              private topBarService: TopBarService) { }

  loadFinished = false;

  logLines = "";

  downloadingLogs : boolean = false;

  ngOnInit() {
    this.toast.clearToastMessages()
    this.getLogData();
    this.topBarService.updateDeviceLocalTime();
  }

  getLogData() {
    this.getSystemInfo.getSystemLog().subscribe(data => {
      //console.log("Received:", data.body);
      switch (data.status) {
        case 200:
          if (data.body !== ''){
            this.logLines = data.body;
            this.loadFinished = true;
          }
          break;
        case 401:
          this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
          break;
        case 500:
          this.httpResponseHandler.handle500HttpErrorResponse();
          break;
        default:
          console.error("Unexpected return code: ", data.status);
      }
    });
  }

  refresh() {
    this.loadFinished = false;
    this.getLogData();
  }

  downloadAllLogs(){
    this.downloadingLogs = true;
    this.getSystemInfo.downloadLogs().subscribe((response: any) => {
      switch (response.status) {
        case 200:
          //console.log(response);
          let dataType = response.body.type;
          //console.log(dataType);
          let filename = response.headers.get('content-disposition').split('=')[1];
          filename = filename.replace(/"/g, '');
          console.log(filename);
          let binaryData = [];
          binaryData.push(response.body);

          let url= window.URL.createObjectURL(new Blob(binaryData, {type: dataType}));
          //window.open(url);

          let anchor = document.createElement("a");
          anchor.download = filename;
          anchor.href = url;
          anchor.click();

          this.downloadingLogs = false;

          //console.log("file downloaded");
          break;
        case 401:
          this.httpResponseHandler.handle401HttpErrorResponse((response as HttpErrorResponse).error);
          break;
        case 500:
          this.httpResponseHandler.handle500HttpErrorResponse();
          break;
        default:
          console.error("Unexpected return code: ", response.status);
      }
    });
  }
}
