import {Component, OnDestroy, OnInit} from '@angular/core';
import {SystemInfoService} from '../services/system-info.service';
import {HttpErrorResponse} from '@angular/common/http';
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {DeviceConfigService} from '../services/device-config.service';
import {TopBarService} from '../services/top-bar.service';
import {ToastrService} from "ngx-toastr";
import {ToastMessageService} from "../services/toast-message.service";
import {timezoneOffsetToString} from "./datetime_utils";
import DeviceTypes from '../utils/DeviceTypes';


@Component({
    selector: 'app-main',
    templateUrl: './main.component.html',
    styleUrls: ['./main.component.css', '../css/general.css']
})


export class MainComponent implements OnInit, OnDestroy {

    dataLoaded: boolean = false;

    mainMenu = [
        {title: 'Settings', routerLink: 'settings'},
        {title: 'Paging Settings', routerLink: 'paging-settings'},
        {title: 'Users', routerLink: 'users'},
        {title: 'Groups', routerLink: 'groups'},
        {title: 'Device Messages', routerLink: 'audio-files'},
        {title: 'System Status', routerLink: 'system-status'},
        {title: 'Logs', routerLink: 'logs'},
        {title: 'Defaults', routerLink: 'defaults'},
        {title: 'Update', routerLink: 'update'},
        {title: 'Reboot', routerLink: 'reboot'}
    ];

    product_name: string = "IC Paging Gateway";

    settingsList = ["local_time", "device_type", "mac_addr", "fw_version"];

    dateTime: number;
    timezoneOffset: string;
    timeSubs = null;
    device_type = "";
    mac_addr = "";
    fw_ver = "";

    constructor(private systemInfoService: SystemInfoService,
                private httpResponseHandler: HTTPResponseHandlerService,
                private configsService: DeviceConfigService,
                private topBarService: TopBarService,
                private toast: ToastMessageService) {
    }

    async ngOnInit() {
        this.toast.clearToastMessages();
        await this.getHeaderInfo();

        this.timeSubs = setTimeout(() => this.updateBarWithDeviceTime(), 1000);
        this.topBarService.device_localtime$.subscribe(async () => {
            let httpResp = await this.systemInfoService.getSystemInfo(["local_time"]).toPromise();
            if (httpResp.status === 200) {
                if (httpResp.body.hasOwnProperty("local_time")) {
                    this.updateDeviceTime(httpResp.body["local_time"][0] * 1000, timezoneOffsetToString(httpResp.body["local_time"][1]));
                    // this.timeCounter = 0;
                }
            }
        });
        this.dataLoaded = true;
    }

    async getHeaderInfo() {
        let httpResp = await this.systemInfoService.getSystemInfo(this.settingsList).toPromise();
        switch (httpResp.status) {
            case 200:
                if (httpResp.body.hasOwnProperty("local_time")) {
                    this.updateDeviceTime(httpResp.body["local_time"][0] * 1000, timezoneOffsetToString(httpResp.body["local_time"][1]));
                }
                if (httpResp.body.hasOwnProperty("device_type")) {
                    this.device_type = httpResp.body["device_type"];
                }
                if (httpResp.body.hasOwnProperty("mac_addr")) {
                    this.mac_addr = "MAC " + httpResp.body["mac_addr"];
                }
                if (httpResp.body.hasOwnProperty("fw_version")) {
                    this.fw_ver = "FW " + httpResp.body["fw_version"];
                }

                break;

            case 401:
                this.httpResponseHandler.handle401HttpErrorResponse((httpResp as HttpErrorResponse).error);
                break;

            case 500:
                this.httpResponseHandler.handle500HttpErrorResponse();
                break;

            default:
                console.error("Unexpected return code: ", httpResp.status);
        }
    }

    async updateBarWithDeviceTime() {
        // this.timeCounter += 1;
        // if (this.timeCounter >= 10) {
        //     let httpResp = await this.systemInfoService.getSystemInfo(["local_time"]).toPromise();
        //     if (httpResp.status === 200) {
        //         if (httpResp.body.hasOwnProperty("local_time")) {
        //             this.updateDeviceTime(httpResp.body["local_time"][0] * 1000, timezoneOffsetToString(httpResp.body["local_time"][1]));
        //             this.timeCounter = 0;
        //         }
        //     } else {
        //         this.dateTime += 1000;
        //     }
        // } else {
        //     this.dateTime += 1000;
        // }
        this.dateTime += 1000;
        this.timeSubs = setTimeout(() => this.updateBarWithDeviceTime(), 1000);
    }

    updateDeviceTime(timestamp, offset) {
        this.dateTime = timestamp;
        this.timezoneOffset = offset;
    }

    ngOnDestroy() {
        if (this.timeSubs) {
            clearInterval(this.timeSubs);
        }
    }

}
