import {Component, Inject, OnInit} from '@angular/core';
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material/dialog";
import {ToastMessageService} from "../services/toast-message.service";
import {FormControl, FormGroup, Validators} from "@angular/forms";

@Component({
  selector: 'app-new-or-edit-group-modal',
  templateUrl: './new-or-edit-group-modal.component.html',
  styleUrls: ['../css/general.css', '../css/new-or-edit-modals.css', './new-or-edit-group-modal.component.css']
})
export class NewOrEditGroupModalComponent implements OnInit {

    constructor(public dialogRef: MatDialogRef<NewOrEditGroupModalComponent>,
                @Inject(MAT_DIALOG_DATA) public data: NewOrEditGroupData,
                private toast: ToastMessageService) { }

    title : string;
    groupForm: FormGroup;

    MAX_CLIENT_ID = 128;
    n_cols : number = 16;
    n_rows: number = 8;
    cols: number[];// = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15];
    rows: number[];// = [0,1,2,3,4,5,6,7];
    all_checked : boolean = false;

    ngOnInit(): void {

        this.cols = Array.from({length: this.n_cols}, (v, k) => k);
        this.rows = Array.from({length: this.n_rows}, (v, k) => k);

        if (this.data.mode === 'new'){
            this.title = "Create new group";
            this.groupForm = new FormGroup<any>({
                "name": new FormControl<string>('', Validators.required),
                "clients": new FormControl<number[]>([], Validators.required)
            });
        } else {
            this.title = "Edit group";
            this.groupForm = new FormGroup<any>({
                "name": new FormControl<string>(this.data.name, Validators.required),
                "clients": new FormControl<number[]>(this.data.clients, Validators.required)
            });
        }
        this.all_checked = this.groupForm.get('clients').value.length === this.MAX_CLIENT_ID;
    }

    submit(){
        if (this.groupForm.valid){
            this.dialogRef.close(this.groupForm.value);
        } else {
            this.toast.showErrorMessage("Group data is invalid!");
        }
    }

    getClientId(row_idx, col_idx){
        return row_idx*this.n_cols+col_idx+1;
    }

    clientIsSelected(clientId){
        return this.groupForm.get('clients').value.indexOf(clientId) > -1;
    }

    selectOrDeselectClient(clientId){
        const idx = this.groupForm.get('clients').value.indexOf(clientId);
        let clientIds = this.groupForm.get('clients').value;
        if(idx > -1){
            clientIds.splice(idx, 1);
        } else {
            clientIds.push(clientId);
        }
        this.groupForm.get('clients').patchValue(clientIds);
        this.all_checked = clientIds.length === this.MAX_CLIENT_ID;
    }

    selectOrDeselectAll(){
        if (this.all_checked){
            this.groupForm.get('clients').patchValue([]);
            this.all_checked = false;
        } else {
            let clientIds = this.groupForm.get('clients').value;
            // select all
            for (let clientId = 1; clientId <= this.MAX_CLIENT_ID; clientId++) {
                if (!this.clientIsSelected(clientId)){
                    clientIds.push(clientId);
                }
            }
            this.groupForm.get('clients').patchValue(clientIds);
            this.all_checked = true;
        }
    }

}

export interface NewOrEditGroupData {
    mode: string; // "new" | "edit"
    name: string | undefined;
    clients: number[];
}
