import {Component, Inject, OnInit} from '@angular/core';
import {FormGroup, FormControl, Validators} from "@angular/forms";
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material/dialog";
import {ToastMessageService} from "../services/toast-message.service";
import {AudioFile} from "../audio-files-tab/audio-files-tab.component";
import {User} from "../users-management-tab/users-management-tab.component";

@Component({
  selector: 'app-new-user',
  templateUrl: './new-or-edit-user.component.html',
  styleUrls: ['../css/general.css', '../css/new-or-edit-modals.css', './new-or-edit-user.component.css']
})
export class NewOrEditUserComponent implements OnInit {

  constructor(public dialogRef: MatDialogRef<NewOrEditUserComponent>,
              @Inject(MAT_DIALOG_DATA) public data: NewOrEditUserData,
              private toast: ToastMessageService) { }

  title : string;
  userForm: FormGroup;

  audio_files_selected : AudioFile[] = [];

  ngOnInit(): void {
      if (this.data.mode === 'new'){
          this.title = "Create new user";
          this.userForm = new FormGroup<any>({
              "name": new FormControl<string>('', Validators.required),
              "email": new FormControl<string>('', Validators.required),
              "audio_files": new FormControl<string[]>([]),
          });
      } else {
          this.getAudioFilesSelected();
          this.title = "Edit user";
          this.userForm = new FormGroup<any>({
              "name": new FormControl<string>(this.data.user.name, Validators.required),
              "email": new FormControl<string>(this.data.user.email, Validators.required),
              "audio_files": new FormControl<string[]>(this.data.user.audio_files ? this.data.user.audio_files : []),
          });
      }
  }

  getAudioFilesSelected() {
      if (this.data.user.audio_files){
          for (let user_audio_file_id of this.data.user.audio_files){
              for (let audio_file of this.data.audio_files){
                  if (audio_file["id"] === user_audio_file_id){
                      this.audio_files_selected.push(audio_file);
                      break;
                  }
              }
          }
      }
  }

  getAudioFileIds(audioFilesList) {
      const audioFileIds = [];
      audioFilesList.forEach( audioFile => {
          audioFileIds.push(audioFile["id"]);
      });
      return audioFileIds;
  }

  submit(){
      if (this.userForm.valid){
          this.dialogRef.close(this.userForm.value);
      } else {
          this.toast.showErrorMessage("User data is invalid!");
      }
  }

  onAudioFilesSelectedChange(selectedAudioFiles: AudioFile[]) {
      this.userForm.get('audio_files').patchValue(this.getAudioFileIds(selectedAudioFiles));
  }

}

export interface NewOrEditUserData {
    mode: string; // "new" | "edit"
    user: User | undefined;
    audio_files: AudioFile[];
}
