import {Component, OnInit, Output, EventEmitter} from '@angular/core';
import {DeviceMessagesService} from "../services/device-messages.service";
import {MatTableDataSource} from "@angular/material/table";
import {v4} from "uuid";
import {ActivatedRoute} from "@angular/router";

const uuid4 = v4;

export interface MessagesInfo {
    btn_name: string;
    file_name: any;
    sample_rate: number;
    timestamp: string;
}

@Component({
    selector: 'app-device-message-body',
    templateUrl: './device-message-body.component.html',
    styleUrls: ['./device-message-body.component.css']
})
export class DeviceMessageBodyComponent  implements OnInit {

    audio: any;
    clickedRowFileName;
    selectedRow;
    first = 0;
    onPlaying = false;
    isPlayingMessageRecorded = null;
    token;
    groupLocalStorage;
    messagePrerecorded = new MatTableDataSource();
    // uuid;
    payload;
    displayedColumns = ['name', 'btn_name'];

    @Output() playingStateChanged = new EventEmitter<boolean>();

    constructor(private deviceMessageService: DeviceMessagesService,
                private route: ActivatedRoute) {
    }

    ngOnInit() {
        this.token = this.route.snapshot.queryParamMap.get('token');
        // this.token = "QT6EhIicocwXkSMFAoBWYg";
        // if (!localStorage.getItem(this.token)) {
        //     localStorage.setItem(this.token, uuid4())
        // } else {
        //     this.uuid = localStorage.getItem(this.token);
        // }
        this.listAllMessages();
        this.groupLocalStorage = JSON.parse(localStorage.getItem('groups'));
        this.messagePrerecorded.filterPredicate = (data: Element, filter: string) => data["btn_name"].trim().toLowerCase().indexOf(filter) != -1;
        this.payload = {
            "groups": this.groupLocalStorage,
            "token": this.token,
            "uuid": uuid4()
        };
        // this.getDeviceMessagePlayingStatus();
    }

    applyFilter(event: KeyboardEvent) {
        const filterValue = (event.target as HTMLInputElement).value;
        this.messagePrerecorded.filter = filterValue.trim().toLowerCase();
    }

    listAllMessages() {
        this.deviceMessageService.getAllDeviceMessages(this.token).subscribe((data) => {
            switch (data.status) {
                case 200:
                    this.messagePrerecorded.data = Object.entries(data.body).map(([id, values]) => ({id, ...values as object}));
                    // console.log("====",Object.values(data.body))
                    break;
                case 404:
                    alert("Bad Request")
                    break;
            }
        })
    }


    // ngOnDestroy() {
    //     console.log("ngOnDestroy");
    //     // this.stopPlayingDeviceMessageOnBrowser();
    //     // this.stopPlayingDeviceMessageOnDevice();
    // }


    async playDeviceMessageOnBrowser(elementName) {
        this.stopPlayingDeviceMessageOnBrowser();
        this.audio = new Audio();

        this.deviceMessageService.getDeviceMessageFile(elementName.file_name, this.token).subscribe((data) => {
            switch (data.status) {
                case 200:
                    let binaryData = [];
                    binaryData.push(data.body);
                    let url = window.URL.createObjectURL(new Blob(binaryData))
                    this.audio.src = url;
                    this.audio.load();
                    this.audio.play();
                    this.audio.onended = () => {
                        this.audio = null;
                        this.isPlayingMessageRecorded = null;
                    }
                    this.isPlayingMessageRecorded = elementName.btn_name;
                    break;
                case 400:
                case 404:
                    alert("Something went wrong!");
                    break;
            }
        });

    }


    stopPlayingDeviceMessageOnBrowser() {
        try {
            if (this.audio) {
                this.audio.pause();
                this.audio = null;
                this.isPlayingMessageRecorded = null;
            }
        } catch (e) {
            console.log(e)
        }
    }


    selectRow(row) {
        this.selectedRow = row;
        this.clickedRowFileName = row.file_name
    }


    stopPlayingDeviceMessageOnDevice() {
        if (this.onPlaying === true) {
            this.deviceMessageService.stopDeviceMessageOnDevice(this.payload, this.token).subscribe((data) => {
                switch (data.status) {
                    case 204:
                        this.onPlaying = false;
                        this.playingStateChanged.emit(this.onPlaying);
                        break;
                    default:
                        let error_msg = "Can not stop playing message!"
                        if (data.error){
                            error_msg += "\n" + data.error;
                        }
                        alert(error_msg);
                        // this.onPlaying = false;
                        break;
                }
            })
        }
    }


    getDeviceMessagePlayingStatus() {
        this.deviceMessageService.getDeviceMessagePlayingStatus(this.payload, this.token).subscribe((data) => {
            switch (data.status) {
                case 200:
                    // this.stopPlayingDeviceMessageOnDevice();
                    this.onPlaying = false;
                    this.playingStateChanged.emit(this.onPlaying);
                    break;
                case 206:
                    this.onPlaying = true;
                    this.playingStateChanged.emit(this.onPlaying);
                    if(this.selectedRow === undefined || this.selectedRow === null){
                        // console.log(data.body.response[0])
                        // console.log(data)
                        this.getSelectedRowById(data.body.response[0]);
                    }
                    setTimeout(() => this.getDeviceMessagePlayingStatus(), 1000);
                    break;
                case 409:
                    this.onPlaying = false;
                    this.playingStateChanged.emit(this.onPlaying);
                    let error_msg = "Error playing message!"
                    if (data.error){
                        error_msg += "\n" + data.error;
                    }
                    alert(error_msg);
                    break;
                default:
                    console.log("Received unexpected status code in getDeviceMessagePlayingStatus ", data.status);
                    // this.stopPlayingDeviceMessageOnDevice();
                    //this.onPlaying = false;
            }
        })
    }


    getSelectedRowById(id: number){
                // @ts-ignore
        this.selectRow(this.messagePrerecorded.data.filter(x => x.id as number == id)[0]);
    }

    playDeviceMessageOnDevice() {
        this.onPlaying = true;
        this.playingStateChanged.emit(this.onPlaying);
        let payload = JSON.parse(JSON.stringify(this.payload));
        payload["file_ID"] = this.selectedRow.file_name;
        this.deviceMessageService.playDeviceMessageOnDevice(payload, this.token).subscribe((data) => {
            switch (data.status) {
                case 204:
                    this.getDeviceMessagePlayingStatus()
                    break;
                default:
                    this.onPlaying = false;
                    this.playingStateChanged.emit(this.onPlaying);
                    // this.stopPlayingDeviceMessageOnDevice()
                    let error_msg = "Can not play message!"
                    if (data.error){
                        error_msg += "\n" + data.error;
                    }
                    alert(error_msg);
                    break;
            }
        })
    }

}
