export class Utils{
    static joinBuffers(channelBuffer, count) {
        let result = new Float64Array(count);
        let offset = 0;
        let lng = channelBuffer.length;

        for (let i = 0; i < lng; i++) {
            let buffer = channelBuffer[i];
            result.set(buffer, offset);
            offset += buffer.length;
        }
        // console.log("result", result)
        return result;
    }


    static convertFloat32ToPcm16(float32Buffer: Float32Array, token: any) {
        // Converts the raw recorded audio from 32bit float to pcm16bit.
        let l = float32Buffer.length + token.length;  //Buffer
        let int16Buffer = new Int16Array(l);
        for (let i = 0; i < token.length; i++) {
            int16Buffer[i] = token.charCodeAt(i);
            //console.log(token.charCodeAt(i))
        }
        for (let i = token.length; i < l; i++) {
            let s = Math.max(-1, Math.min(1, float32Buffer[i - token.length]));
            int16Buffer[i] = s < 0 ? s * 0x8000 : s * 0x7FFF;
            // int16Buffer[idx] = float32Buffer[idx - my_id.length]
        }
        return int16Buffer;
    }


    static mergeBuffers(channelBuffer, recordingLength) {
        let result = new Float32Array(recordingLength);
        let offset = 0;

        for (let i = 0; i < channelBuffer.length; i++) {
            result.set(channelBuffer[i], offset);
            offset += channelBuffer[i].length;
        }

        return Array.prototype.slice.call(result);
    }

    static formatTime(seconds) {
        const h = Math.floor(seconds / 3600)
        const m = Math.floor((seconds % 3600) / 60)
        const s = seconds % 60
        return [h, m > 9 ? m : h ? '0' + m : m || '0', s > 9 ? s : '0' + s]
            .filter(a => a)
            .join(':')
    }





    static writeUTFBytes(view, offset, string) {
        const lng = string.length;
        for (let i = 0; i < lng; i++) {
            view.setUint8(offset + i, string.charCodeAt(i));
        }
    }


    static exportWav(recordedData,recordingLength,smp_rate) {
        let data = this.joinBuffers(recordedData, recordingLength)
        let dataLength = data.length;
        // create wav file
        const buffer = new ArrayBuffer(44 + dataLength * 2);
        let view = new DataView(buffer);

        this.writeUTFBytes(view, 0, 'RIFF'); // RIFF chunk descriptor/identifier
        view.setUint32(4, 44 + dataLength * 2, true); // RIFF chunk length
        this.writeUTFBytes(view, 8, 'WAVE'); // RIFF type
        this.writeUTFBytes(view, 12, 'fmt '); // format chunk identifier, FMT sub-chunk
        view.setUint32(16, 16, true); // format chunk length
        view.setUint16(20, 1, true); // sample format (raw)
        view.setUint16(22, 1, true); // mono (1 channel)
        view.setUint32(24, smp_rate, true); // sample rate
        view.setUint32(28, smp_rate * 2, true); // byte rate (sample rate * block align)
        view.setUint16(32, 2, true); // block align (channel count * bytes per sample)
        view.setUint16(34, 16, true); // bits per sample
        this.writeUTFBytes(view, 36, 'data'); // data sub-chunk identifier
        view.setUint32(40, dataLength * 2, true); // data chunk length
        let index = 44

        for (let i = 0; i < data.length; i++) {
            view.setInt16(index, data[i] * 0x7FFF, true);
            index += 2;
        }
        return{"buffer": buffer, "view": view}
    }

}
