import {Component, OnInit} from '@angular/core';
import {GroupsService} from "../services/groups.service";
import {MessagesInfo} from "../device-message-body/device-message-body.component";
import {MatTableDataSource} from "@angular/material/table";
import {ActivatedRoute} from '@angular/router';

@Component({
    selector: 'app-paging-group-list',
    templateUrl: './paging-group-list.component.html',
    styleUrls: ['./paging-group-list.component.css'],

})
export class PagingGroupListComponent implements OnInit {
    selectedRow: boolean;
    allSelected
    token;
    groupArrayList = new MatTableDataSource();


    selectedOptions: string[] = [];

    constructor(private groupsService: GroupsService,
                private route: ActivatedRoute) {
    }

    ngOnInit(): void {
        this.token = this.route.snapshot.queryParamMap.get('token')
        this.groupsService.getAllGroups(this.token).subscribe((data) => {
            switch (data.status) {
                case 200:
                    let groupsList = Object.entries(data.body).map(([id, values]) => ({id, ...values as object}));
                    // console.log(groupsList)
                    for (let i = 0; i < groupsList.length; i++) {
                        for (let j = 0; j < (groupsList.length - i - 1); j++) {
                            if (groupsList[j]["name"] === "") {
                                groupsList[j]["name"] = ("Group " + groupsList[j]["id"].toString());
                            } else {
                                groupsList[j]["name"] = groupsList[j]["name"];
                            }
                            if (groupsList[j + 1]["name"] === "") {
                                groupsList[j + 1]["name"] = ("Group " + groupsList[j + 1]["id"].toString());
                            } else {
                                groupsList[j + 1]["name"] = groupsList[j + 1]["name"];
                            }
                            if ( parseInt(groupsList[j]["id"]) > parseInt(groupsList[j + 1]["id"])) {
                                let temp = groupsList[j]
                                groupsList[j] = groupsList[j + 1]
                                groupsList[j + 1] = temp
                            }
                        }
                    }
                    this.groupArrayList.data = groupsList;
                    if (localStorage.getItem('groups')) {
                        this.selectedOptions = JSON.parse(localStorage.getItem('groups'));
                        this.verifySelectAll()
                    }
                    break;
                default:
                    alert("Fetch Groups Failed")
            }
        })


        this.groupArrayList.filterPredicate = (data: Element, filter: string) => data["name"].trim().toLowerCase().indexOf(filter) != -1;
    }

    applyFilter(event: KeyboardEvent) {
        const filterValue = (event.target as HTMLInputElement).value;
        this.groupArrayList.filter = filterValue.trim().toLowerCase();
    }


    onAreaListControlChanged(group: any) {
        if (this.selectedOptions.includes(group.id)) {
            for (let i = 0; i < this.selectedOptions.length; i++) {
                if (this.selectedOptions[i] === group.id) {
                    this.selectedOptions.splice(i, 1);
                    localStorage.setItem('groups', JSON.stringify(this.selectedOptions))
                }
            }
        } else {
            this.selectedOptions.push(group.id);
            localStorage.setItem('groups', JSON.stringify(this.selectedOptions))
        }
        this.verifySelectAll()
    }

    onCheckBoxAllEventChange() {
        if (this.allSelected) {
            this.unselectAll();
        } else {
            this.selectAll();
        }
    }

    selectAll() {
        this.selectedOptions = [];
        this.allSelected = true;
        for (let i = 0; i < this.groupArrayList.data.length; i++) {
            this.selectedOptions.push(this.groupArrayList.data[i]["id"]);
        }
        localStorage.setItem('groups', JSON.stringify(this.selectedOptions))
    }

    unselectAll() {
        this.allSelected = false;
        this.selectedOptions = [];
        localStorage.setItem('groups', JSON.stringify(this.selectedOptions))
    }

    verifySelectAll() {
        if (this.selectedOptions.length === this.groupArrayList.data.length) {
            this.allSelected = true
        } else {
            this.allSelected = false
        }
    }

    displayedColumns = ['name'];
    clickedRows = new Set<MessagesInfo>();
}

