import {Injectable} from '@angular/core';
import ServiceUtils from "../../services/service.utils";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {catchError, Observable, of} from "rxjs";
import {environment} from "../../../environments/environment";

@Injectable({
    providedIn: 'root'
})
export class DeviceMessagesService {

    constructor(private http: HttpClient) {
    }

    private readonly PAGING_ENDPOINT = environment.pagingEndpoint;
    // private readonly DEVICE_MESSAGES_ENDPOINT = environment.deviceMessagesEndpoint;

    buildHttpOptions(): any {
        return {
            headers: new HttpHeaders({
                "Content-Type": "application/json"
            }),
            //responseType: "blob" as "blob",
            observe: "response" as "response" // in order to return full response

        }
    }

    getAllDeviceMessages(token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT + '/device-messages?token=' + token;
        return this.http.get<any>(url, {observe: "response" as "response"}).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }

    getDeviceMessageFile(name: string, token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT + "/device-messages/"+ name + "/file?token=" + token;
        return this.http.get<any>(url,{observe: "response" as "response", responseType: "blob" as "json"}).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }

    playDeviceMessageOnDevice(payload: object, token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT+ '/device-messages/play-file?token='+token;
        return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }

    stopDeviceMessageOnDevice(payload: object, token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT+ '/device-messages/stop-file?token='+token;
        return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }

    getDeviceMessagePlayingStatus(payload: object, token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT + '/device-messages/play-status?token='+token;
        return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }


}
