import {Injectable} from '@angular/core';
import {HttpClient, HttpHeaders} from "@angular/common/http";
import ServiceUtils from "../../services/service.utils";
import {catchError, Observable} from "rxjs";
import {environment} from "../../../environments/environment";

@Injectable({
    providedIn: 'root'
})
export class PagingService {

    constructor(private http: HttpClient) {
    }


    private readonly PAGING_ENDPOINT = environment.pagingEndpoint;
    private readonly RECORDING_ENDPOINT = environment.recordingEndpoint;

    buildHttpOptions(): any {
        return {
            headers: new HttpHeaders({
                "Content-Type": "application/json"
            }),
            //responseType: "blob" as "blob",
            observe: "response" as "response" // in order to return full response
        }
    }

    startPaging(payload: object, token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT + '/start?token=' + token;
        return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }


    stopPaging(payload: object, token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT + '/stop?token=' + token;
        return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }


    page(payload: any, token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT + '/page?token=' + token;
        return this.http.post<any>(url, payload, {
            headers: new HttpHeaders({
                "Content-Type": "text/plain"
            }), observe: "response" as "response"
        }).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }

    // startOrStopPaging(payload: object, token: string): Observable<any> {
    //     const url = this.PAGING_ENDPOINT + '?token=' + token;
    //     return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
    //         catchError(err1 => ServiceUtils.handleError(err1))
    //     );
    // }

    // startOrStopRecording(payload: object, token: string): Observable<any> {
    //     const url = this.RECORDING_ENDPOINT + '?token=' + token;
    //     return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
    //         catchError(err1 => ServiceUtils.handleError(err1))
    //     );
    // }

    startPagingRecorded(payload: object, token: string): Observable<any> {
        const url = this.RECORDING_ENDPOINT + '/start-paging?token=' + token;
        return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }

    stopPagingRecorded(payload: object, token: string): Observable<any> {
        const url = this.RECORDING_ENDPOINT + '/stop-paging?token=' + token;
        return this.http.post<any>(url, payload, this.buildHttpOptions()).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }

    pageRecorded(payload, token: string): Observable<any> {
        const url = this.RECORDING_ENDPOINT + '/page?token=' + token;
        return this.http.post<any>(url, payload, {
            headers: new HttpHeaders({
                "Accept": "*/*",
                "Content-Type": "text/plain"
            }), observe: "response" as "response"
        }).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }

    //PUT Paging for live record
    // paging(payload: any, token: string): Observable<any> {
    //     const url = this.PAGING_ENDPOINT + '?token=' + token;
    //     return this.http.put<any>(url, payload, {
    //         headers: new HttpHeaders({
    //             "Content-Type": "text/plain"
    //         }), observe: "response" as "response"
    //     }).pipe(
    //         catchError(err1 => ServiceUtils.handleError(err1))
    //     );
    // }

    // recording(payload, token: string): Observable<any> {
    //     const url = this.RECORDING_ENDPOINT + '?token=' + token;
    //     return this.http.put<any>(url, payload, {
    //         headers: new HttpHeaders({
    //             "Accept": "*/*",
    //             "Content-Type": "text/plain"
    //         }), observe: "response" as "response"
    //     }).pipe(
    //         catchError(err1 => ServiceUtils.handleError(err1))
    //     );
    // }

    getSampleRate(token: string): Observable<any> {
        const url = this.PAGING_ENDPOINT + '/sample-rate?token=' + token;
        return this.http.get<any>(url, {
            headers: new HttpHeaders({"Accept": "*/*", "Content-Type": "text/plain"}),
            observe: "response" as "response"
        }).pipe(
            catchError(err1 => ServiceUtils.handleError(err1))
        );
    }
}
