import {Component, OnInit} from '@angular/core';
import {DeviceConfigService} from "../services/device-config.service";
import {SystemFunctionsService} from "../services/system-functions.service";
import {HttpErrorResponse} from '@angular/common/http';
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {interval} from 'rxjs';
import {ToastrService} from "ngx-toastr";
import {ToastMessageService} from "../services/toast-message.service";
import { TopBarService } from '../services/top-bar.service';


@Component({
    selector: 'app-reboot',
    templateUrl: './reboot.component.html',
    styleUrls: ['./reboot.component.css', '../css/general.css']
})


export class RebootComponent implements OnInit {

    loadFinished = false;
    reset_opt = false;
    rebooting = false;
    waitingMessage: string;

    constructor(private configsService: DeviceConfigService,
                private systemFuncsService: SystemFunctionsService,
                private httpResponseHandler: HTTPResponseHandlerService,
                private toast: ToastMessageService,
                private topBarService: TopBarService) {
    }

    ngOnInit() {
        this.toast.clearToastMessages()
        this.configsService.getUcis({'defaults_opt': 'security.reset.enabled'}).subscribe(data => {
            switch (data.status) {
                case 200:
                    //build device form group
                    if (data.body["security.reset.enabled"] === "true") {
                        this.reset_opt = true;
                    }
                    break;
                case 401:
                    this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
                    break;
                case 500:
                    this.httpResponseHandler.handle500HttpErrorResponse();
                    break;
                default:
                    console.error("Unexpected return code: ", data.status);
            }
            this.loadFinished = true;
        });
        this.topBarService.updateDeviceLocalTime();
    }

    rebootSystem() {
        this.systemFuncsService.reboot().subscribe(data => {
            //console.log("Status:", data.status);
            //console.log("Received:", data.body);
            switch (data.status) {
                case 200:
                    this.waitingMessage = "Please wait while the device reboots. The page will be automatically reloaded.";
                    this.rebooting = true;
                    setTimeout(() => this.reloadWebUI(), 20000);
                    break;
                case 401:
                    this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
                    break;
                case 500:
                    this.httpResponseHandler.handle500HttpErrorResponse();
                    break;
                default:
                    console.error("Unexpected return code: ", data.status);
            }
        });
    }

    reloadWebUI() {
        this.systemFuncsService.appIsRunning().subscribe(res => {
            if (res.status === 200) {
                window.location.href = window.location.protocol + '//' + window.location.hostname;
            } else {
                setTimeout(() => this.reloadWebUI(), 1000);
            }
        });
    }

}
