import {Component, Input, OnInit, Output, EventEmitter} from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";

@Component({
  selector: 'app-select-from-list',
  templateUrl: './select-from-list.component.html',
  styleUrls: ['../css/general.css', './select-from-list.component.css']
})
export class SelectFromListComponent implements OnInit {

    @Input() display_label : string;
    @Input() completeListTableHeader : string;
    @Input() completeList : any[];
    @Input() selectedList : any[];
    @Input() selectedListTableHeader : string;
    @Output() onSelectedListChange = new EventEmitter<any>();

    availableListDisplayedColumns = ['display_label'];
    selectedListDisplayedColumns = ['display_label'];
    availableList = [];
    availableListDS : MatTableDataSource<any>;
    selectedListDS : MatTableDataSource<any>;

    show : boolean = false;

  constructor() { }

  ngOnInit(): void {
      this.getListWithoutSelected();
      this.availableListDS = new MatTableDataSource<any>(this.availableList);
      this.selectedListDS = new MatTableDataSource<any>(this.selectedList);
      this.show = true;
  }

  getListWithoutSelected() {
      this.completeList.forEach((item) => {
          if (!this.selectedList.includes(item)){
              this.availableList.push(item);
          }
      });
  }

  applyFilter(event: Event, dataSource: MatTableDataSource<any>) {
      const filterValue = (event.target as HTMLInputElement).value;
      dataSource.filter = filterValue.trim().toLowerCase();
  }

  selectElem(elem) {
      if (!this.selectedList.includes(elem)) {
          this.selectedList.push(elem);
          const idx = this.availableList.indexOf(elem);
          if (idx > -1){
              this.availableList.splice(idx, 1);
          }
          this.onSelectedListChange.emit(this.selectedList);

          this.availableListDS.data = this.availableList;
          this.selectedListDS.data = this.selectedList;
      }
  }

  deselectElem(elem) {
      const idx = this.selectedList.indexOf(elem);
      if (idx > -1){
          this.selectedList.splice(idx, 1);
          this.availableList.push(elem);
          this.onSelectedListChange.emit(this.selectedList);

          this.availableListDS.data = this.availableList;
          this.selectedListDS.data = this.selectedList;
      }
  }

  selectAll() {
      for (let elem of this.availableList){
          this.selectedList.push(elem);
          this.availableList = [];
      }
      this.onSelectedListChange.emit(this.selectedList);

      this.availableListDS.data = this.availableList;
      this.selectedListDS.data = this.selectedList;
  }

  deselectAll() {
      for (let elem of this.selectedList){
          this.availableList.push(elem);
          this.selectedList = [];
      }
      this.onSelectedListChange.emit(this.selectedList);

      this.availableListDS.data = this.availableList;
      this.selectedListDS.data = this.selectedList;
  }

}
