import { Injectable } from '@angular/core';
import {HttpClient, HttpErrorResponse, HttpHeaders, HttpResponse} from '@angular/common/http';
import {environment} from '../../environments/environment';
import {catchError, map, tap} from 'rxjs/operators';
import {Observable, of} from 'rxjs';


const LOGIN_ENDPOINT = environment.loginEndpoint;

@Injectable({
  providedIn: 'root'
})
export class AccountService {

  constructor(private httpClient: HttpClient) { }

  private httpOptions = {
    headers: new HttpHeaders({
      'Content-Type':  'application/json'
    }),
    observe: 'response' as 'response',
    responseType: 'text' as 'json'
  };

  login(username, password) {
    let dict = { 'username': username, 'password': password }
    return this.httpClient.post(LOGIN_ENDPOINT, JSON.stringify(dict), this.httpOptions).pipe(
        tap(res => {
          localStorage.setItem('loginState', 'isLoggedIn');
          return of(res);
        }),
        catchError(err1 => this.handleError(err1))
    );
  }


  isLoggedIn$(): Observable<boolean> {
    if (localStorage.getItem('loginState') === 'isLoggedIn'){
      return of(true);
    } else {
      return of(false);
    }
  }

  private handleLoginError(error: HttpErrorResponse) {
    //console.log(error);
    if (error.error instanceof ErrorEvent) {
      // A client-side or network error occurred. Handle it accordingly.
      console.error('An error occurred:', error.error.message);
    } else {
      if (error.status === 200){
        //this.loggedIn = true;
        //console.log(error.headers);
        //console.log(error.headers.get("session"));
        //this.cookieService.set('sessionID', error.headers['Cookie'])
        return of(new HttpResponse({body: error.message, headers: error.headers, status: error.status, url: error.url}));
      } else {
        console.error(
            `Backend returned code ${error.status}, ` +
            `body was: ${error.error}`);
      }
    }
    return of(error);
  }

  private handleError(error: HttpErrorResponse) {
    //console.log(error);
    if (error.error instanceof ErrorEvent) {
      // A client-side or network error occurred. Handle it accordingly.
      console.error('An error occurred:', error.error.message);
    } else {
      // The backend returned an unsuccessful response code.
      // The response body may contain clues as to what went wrong.
      console.error(
          `Backend returned code ${error.status}, ` +
          `body was: ${error.error}`);
    }
    // Return an observable with a user-facing error message.
    //return throwError('Something bad happened; please try again later.');
    return of(error); //returns HttpErrorResponse with status 0
  }
}
