import { Injectable } from '@angular/core';
import {Observable, of} from 'rxjs';
import {environment} from '../../environments/environment';
import {HttpClient, HttpErrorResponse, HttpHeaders} from '@angular/common/http';
import {catchError} from 'rxjs/operators';

const CERTIFICATES_ENDPOINT = environment.certificatesEndpoint;

@Injectable({
  providedIn: 'root'
})
export class AdditionalCertificatesService {

  constructor(private httpClient: HttpClient) { }

  private httpOptions = {
    observe: 'response' as 'response'
  };

  upload(file):Observable<any> {

    //console.log(file);

    // Create form data
    let formData = new FormData();

    formData.append('certificate', file, file.name);

    //console.log(formData.getAll('certificate'));
    // Make http post request over api
    // with formData as req
    return this.httpClient.post(CERTIFICATES_ENDPOINT,formData, {observe: 'response', responseType: 'text'}).pipe(
        /*switchMap(res => {
          console.log(res.status);
          return of(res);
        }),*/
        catchError(err1 => this.handleError(err1)));
  }

  get(): Observable<any> {
    //console.log('CERTIFICATES_ENDPOINT get', CERTIFICATES_ENDPOINT);
    return this.httpClient.get<any>(CERTIFICATES_ENDPOINT, this.httpOptions).pipe(
        /*switchMap(res => {
          console.log(res.status);
          return of(res);
        }),*/
        catchError(err1 => this.handleError(err1)));
  }

  delete(certificateList): Observable<any> {
    //console.log('CERTIFICATES_ENDPOINT delete', CERTIFICATES_ENDPOINT);
    let options: Object = {
      headers: new HttpHeaders({
        'Content-Type':  'application/json'
      }),
      responseType: 'text',
      observe: 'response' as 'response',
      body: {
        'payload': certificateList
      }
    };
    return this.httpClient.delete<any>(CERTIFICATES_ENDPOINT, options).pipe(
        /*switchMap(res => {
          console.log(res.status);
          return of(res);
        }),*/
        catchError(err1 => this.handleError(err1)));
  }

  private handleError(error: HttpErrorResponse) {
    if (error.error instanceof ErrorEvent) {
      // A client-side or network error occurred. Handle it accordingly.
      console.error('An error occurred:', error.error.message);
    } else {
      // The backend returned an unsuccessful response code.
      // The response body may contain clues as to what went wrong.
      console.error(
          `Backend returned code ${error.status}, ` +
          `body was: ${error.error}`);
    }
    // Return an observable with a user-facing error message.
    //return throwError('Something bad happened; please try again later.');
    return of(error); //returns HttpErrorResponse with status 0
  }
}
