import { Injectable } from '@angular/core';
import {environment} from "../../environments/environment";
import {HttpClient, HttpRequest} from "@angular/common/http";
import {catchError} from "rxjs/operators";
import {Observable} from "rxjs";
import ServiceUtils from "./service.utils";

@Injectable({
  providedIn: 'root'
})
export class AudioFilesService {

    AUDIO_FILES_ENDPOINT = environment.audioFilesEndpoint;

    constructor(private httpClient: HttpClient) { }

    getAudioFiles(): Observable<any> {
        return this.httpClient.get(this.AUDIO_FILES_ENDPOINT, {observe: 'response' as 'response', responseType: 'text'})
            .pipe(
                catchError(err1 => ServiceUtils.handleError(err1)));
    }

    saveNewAudioFile(formData): Observable<any> {

        const req = new HttpRequest('POST', this.AUDIO_FILES_ENDPOINT, formData, {
            reportProgress: true,
            responseType: 'text',
            /*headers: new HttpHeaders({
              'rejectUnauthorized': 'false'
            })*/
        });

        return this.httpClient.request(req);

        // return this.httpClient.post(this.AUDIO_FILES_ENDPOINT, formData, {
        //     headers: {'Content-Type': 'application/json'},
        //     observe: 'response' as 'response',
        //     responseType: 'text'})
        //     .pipe(
        //         catchError(err1 => ServiceUtils.handleError(err1)));
    }

    saveAudioFileChanges(formData): Observable<any> {

        const req = new HttpRequest('PUT', this.AUDIO_FILES_ENDPOINT, formData, {
            reportProgress: true,
            responseType: 'text',
            /*headers: new HttpHeaders({
              'rejectUnauthorized': 'false'
            })*/
        });

        return this.httpClient.request(req);

        // return this.httpClient.put(this.AUDIO_FILES_ENDPOINT, formData, {
        //     reportProgress: true,
        //     observe: 'response' as 'response',
        //     responseType: 'text'});
    }

    deleteAudioFiles(userIds): Observable<any> {
        return this.httpClient.delete(this.AUDIO_FILES_ENDPOINT, {
            body: userIds,
            headers: {'Content-Type': 'application/json'},
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => ServiceUtils.handleError(err1)));
    }

    uploadPregong(file):Observable<any> {

        // Create form data
        let formData = new FormData();

        formData.append('file', file, file.name);

        return this.httpClient.post(this.AUDIO_FILES_ENDPOINT + "/pregong",formData, {observe: 'response', responseType: 'text'}).pipe(
            catchError(err1 => ServiceUtils.handleError(err1)));
    }

    getPregongData():Observable<any> {
        return this.httpClient.get(this.AUDIO_FILES_ENDPOINT + "/pregong/info", {observe: 'response' as 'response', responseType: 'text'})
            .pipe(
                catchError(err1 => ServiceUtils.handleError(err1)));
    }

    deletePregong(): Observable<any> {
        return this.httpClient.delete(this.AUDIO_FILES_ENDPOINT + "/pregong", {
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => ServiceUtils.handleError(err1)));
    }
}
