import { Injectable } from '@angular/core';
import {Observable, of} from "rxjs";
import {HttpClient, HttpErrorResponse, HttpHeaders} from "@angular/common/http";
import {environment} from "../../environments/environment";
import {catchError, tap} from "rxjs/operators";

const DEVICE_UCIS_ENDPOINT = environment.deviceUcisEndpoint;
const FACTORY_DEFAULTS_ENDPOINT = environment.resetFactoryDefaultsEndpoint;
const SETTINGS_ENDPOINT = environment.settingsEndpoint;
const VOLUME_ENDPOINT = environment.volumeControlEndpoint;
const MUTED_ENDPOINT = environment.mutedControlEndpoint;

@Injectable({
  providedIn: 'root'
})
export class DeviceConfigService {

  constructor(private httpClient: HttpClient) { }

  private httpOptions = {
    headers: new HttpHeaders({
      'Content-Type':  'application/json'
    }),
    observe: 'response' as 'response'
  };

  getUcis(ucisList:{}): Observable<any> {

    let values = '';
    let isFirst = true;

    Object.keys(ucisList).forEach((key) => {
      if (isFirst)  {
        isFirst = false;
      } else {
        values += '&';
      }
      values += ucisList[key];
    });


    const url = DEVICE_UCIS_ENDPOINT + '?' + values;
    //console.log('REST UCI get "%s"', url);
    return this.httpClient.get<any>(url, this.httpOptions).pipe(
      /*switchMap(res => {
        console.log(res.status);
        return of(res);
      }),*/
      catchError(err1 => this.handleError(err1)));
  }

  setSettings(payload): Observable<any> {
    //console.log(JSON.stringify(payload));

    return this.httpClient.put<any>(SETTINGS_ENDPOINT, JSON.stringify(payload), this.httpOptions)
      .pipe(
        /*switchMap(res => {
          console.log(res.status);
          return of(res);
        }),*/
        catchError(err1 => this.handleError(err1)));
  }

  resetFactoryDefaults(): Observable<any> {
    //console.log('FACTORY_DEFAULTS_ENDPOINT get "%s"', FACTORY_DEFAULTS_ENDPOINT);
    return this.httpClient.post(FACTORY_DEFAULTS_ENDPOINT, {}, {observe: 'response' as 'response'})
      .pipe(
        catchError(err1 => this.handleError(err1)));
  }

  setVolume(uciDict) : Observable<any> {
    return this.httpClient.put<any>(VOLUME_ENDPOINT, JSON.stringify(uciDict), this.httpOptions)
        .pipe(
            /*switchMap(res => {
              console.log(res.status);
              return of(res);
            }),*/
            catchError(err1 => this.handleError(err1)));
  }

  setMuted(uciDict): Observable<any> {
    return this.httpClient.put<any>(MUTED_ENDPOINT, JSON.stringify(uciDict), this.httpOptions)
        .pipe(catchError(err1 => this.handleError(err1)));
  }

  private handleError(error: HttpErrorResponse) {
    if (error.error instanceof ErrorEvent) {
      // A client-side or network error occurred. Handle it accordingly.
      console.error('An error occurred:', error.error.message);
    } else {
      // The backend returned an unsuccessful response code.
      // The response body may contain clues as to what went wrong.
      console.error(
        `Backend returned code ${error.status}, ` +
        `body was: ${error.error}`);
    }
    // Return an observable with a user-facing error message.
    //return throwError('Something bad happened; please try again later.');
    return of(error); //returns HttpErrorResponse with status 0
  }

}
