import { Injectable } from '@angular/core';
import {environment} from "../../environments/environment";
import {HttpClient, HttpErrorResponse} from "@angular/common/http";
import {catchError} from "rxjs/operators";
import {Observable, of} from "rxjs";

@Injectable({
  providedIn: 'root'
})
export class GroupsManagementService {

    GROUPS_ENDPOINT = environment.groupsEndpoint;

    constructor(private httpClient: HttpClient) { }

    getGroups(): Observable<any> {
        return this.httpClient.get(this.GROUPS_ENDPOINT, {observe: 'response' as 'response', responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    saveNewGroup(groupData): Observable<any> {
        return this.httpClient.post(this.GROUPS_ENDPOINT, groupData, {
            headers: {'Content-Type': 'application/json'},
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    saveGroupChanges(groupData): Observable<any> {
        return this.httpClient.put(this.GROUPS_ENDPOINT, groupData, {
            headers: {'Content-Type': 'application/json'},
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    deleteGroups(groupIds): Observable<any> {
        return this.httpClient.delete(this.GROUPS_ENDPOINT, {
            body: groupIds,
            headers: {'Content-Type': 'application/json'},
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    private handleError(error: HttpErrorResponse) {
        if (error.error instanceof ErrorEvent) {
            // A client-side or network error occurred. Handle it accordingly.
            console.error('An error occurred:', error.error.message);
        } else {
            // The backend returned an unsuccessful response code.
            // The response body may contain clues as to what went wrong.
            console.error(
                `Backend returned code ${error.status}, ` +
                `body was: ${error.error}`);
        }
        // Return an observable with a user-facing error message.
        //return throwError('Something bad happened; please try again later.');
        return of(error); //returns HttpErrorResponse with status 0
    }
}
