import {Injectable} from '@angular/core';
import {HttpErrorResponse} from '@angular/common/http';
import {ToastrService} from 'ngx-toastr';
import {Router} from '@angular/router';
import {ToastMessageService} from "./toast-message.service";


@Injectable({
    providedIn: 'root'
})


export class HTTPResponseHandlerService {

    constructor(private toastr: ToastrService,
                private toast: ToastMessageService,
                private router: Router) {
    }

    handle401HttpErrorResponse(msg) {
        if (msg === "Login required") {
            this.router.navigate(['/login']);
        } else if (msg === "Invalid username or password") {
            this.toastr.error(msg, '', {
                closeButton: true,
                timeOut: 1000000
            });
        } else if (msg === "Session expired") {
            let sessionExpireToasts = document.getElementsByClassName("session-expired");
            if (sessionExpireToasts.length === 0) { //prevent from showing multiple session expired toasts
                this.toastr.error(msg, '', {
                    closeButton: true,
                    timeOut: 10000
                });
                this.router.navigate(['/login']);
            }
        }
    }

    handle500HttpErrorResponse() {
        let error_msg = "Internal error! Please try again or reboot your device if the error persists."
        this.toast.showErrorMessage(error_msg);
    }

    handleUnexpectedHttpErrorResponse(code) {
        let error_msg = "An unexpected HTTP Status code was received: " + code + ". Please try again or reboot your device if error persists."
        this.toast.showErrorMessage(error_msg);
    }
}
