import {Injectable} from '@angular/core';
import {HttpClient, HttpErrorResponse, HttpHeaders} from "@angular/common/http";
import {Observable, of} from "rxjs";
import {catchError, tap, timeout} from 'rxjs/operators';
import {environment} from "../../environments/environment";

const REBOOT_ENDPOINT = environment.rebootEndpoint;
const SYSTEM_ENDPOINT = environment.systemInfoEndpoint;

@Injectable({
    providedIn: 'root'
})
export class SystemFunctionsService {

    constructor(private httpClient: HttpClient) {
    }

    private httpOptions = {
        headers: new HttpHeaders({
            'Content-Type': 'application/json'
        }),
        observe: 'response' as 'response'
    };

    reboot(): Observable<any> {
        //console.log('REBOOT_ENDPOINT get "%s"', REBOOT_ENDPOINT);
        return this.httpClient.post(REBOOT_ENDPOINT, {}, this.httpOptions).pipe(
            /*switchMap(res => {
              console.log(res.status);
              return of(res);
            }),*/
            catchError(err1 => this.handleError(err1)));
    }

    appIsRunning(): Observable<any> {
        //console.log('Home endpoint get "%s"', HOME_ENDPOINT);
        return this.httpClient.get<any>(SYSTEM_ENDPOINT, {observe: 'response' as 'response'}).pipe(
            catchError(err1 => this.handleError(err1)));
    }

    private handleError(error: HttpErrorResponse) {
        if (error.error instanceof ErrorEvent) {
            // A client-side or network error occurred. Handle it accordingly.
            console.error('An error occurred:', error.error.message);
        } else {
            // The backend returned an unsuccessful response code.
            // The response body may contain clues as to what went wrong.
            console.error(
                `Backend returned code ${error.status}, ` +
                `body was: ${error.error}`);
        }
        // Return an observable with a user-facing error message.
        //return throwError('Something bad happened; please try again later.');
        return of(error); //returns HttpErrorResponse with status 0
    }
}
