import { Injectable } from '@angular/core';
import {Observable, of} from "rxjs";
import {environment} from "../../environments/environment";
import {HttpClient, HttpErrorResponse, HttpHeaders, HttpResponse} from '@angular/common/http';
import {catchError, map, tap} from "rxjs/operators";


//const HEADER_SYS_INFO_ENDPOINT = environment.headerSysInfoEndpoint;
//const FW_INFO_ENDPOINT = environment.fwInfoEndpoint;
//const SYS_STATUS_ENDPOINT = environment.systemStatusEndpoint;
const SYS_LOG_ENDPOINT = environment.systemLogEndpoint;
const DOWNLOAD_LOGS_ENDPOINT = environment.downloadLogsEndpoint;
const SYS_INFO_ENDPOINT = environment.systemInfoEndpoint;
const SYS_TYPE_ENDPOINT = environment.systemTypeEndpoint;

/* AES67 specific status */

@Injectable({
  providedIn: 'root'
})
export class SystemInfoService {

  constructor(private httpClient: HttpClient) { }

  private httpOptions = {
    headers: new HttpHeaders({
      'Content-Type':  'application/json'
    }),
    observe: 'response' as 'response'
  };

  // getHeaderSystemInfo(): Observable<any> {
  //   //console.log('HEADER_SYS_INFO_ENDPOINT get "%s"', HEADER_SYS_INFO_ENDPOINT);
  //   return this.httpClient.get(HEADER_SYS_INFO_ENDPOINT, {observe: 'response' as 'response', responseType: 'text'})
  //     .pipe(
  //       /*tap( // Log the result or error
  //         data => console.log(data),
  //         error => console.log("Error!", error)
  //       )*/
  //       catchError(err1 => this.handleError(err1)));
  // }

  // getCurrentFWInfo(): Observable<any> {
  //   //console.log('HEADER_SYS_INFO_ENDPOINT get "%s"', FW_INFO_ENDPOINT);
  //   return this.httpClient.get<any>(FW_INFO_ENDPOINT, this.httpOptions).pipe(
  //     /*switchMap(res => {
  //       console.log(res.status);
  //       return of(res);
  //     }),*/
  //     catchError(err1 => this.handleError(err1)));
  // }

  // getSystemStatus(): Observable<any> {
  //   //console.log('SYS_STATUS_ENDPOINT get "%s"', SYS_STATUS_ENDPOINT);
  //   return this.httpClient.get<any>(SYS_STATUS_ENDPOINT, this.httpOptions).pipe(
  //     /*switchMap(res => {
  //       console.log(res.status);
  //       return of(res);
  //     }),*/
  //     catchError(err1 => this.handleError(err1)));
  // }

  getSystemLog(): Observable<any> {
    //console.log('SYS_LOG_ENDPOINT get "%s"', SYS_LOG_ENDPOINT);
    return this.httpClient.get(SYS_LOG_ENDPOINT, {observe: 'response' as 'response', responseType: 'text'})
      .pipe(
        /*tap( // Log the result or error
          //data => console.log(data),
          error => console.log("Error!", error)
        )*/
        catchError(err1 => this.handleError(err1)));
  }

  downloadLogs():Observable<any>{
    //console.log('DOWNLOAD_LOGS_ENDPOINT get "%s"', DOWNLOAD_LOGS_ENDPOINT);
    /*let requestOptions = {
        headers: new HttpHeaders({
          'Content-Type':  'application/x-tar'
        }),
        observe: 'response' as 'response'
      };
    return this.httpClient.get(DOWNLOAD_LOGS_ENDPOINT, requestOptions);*/
    //return this.httpClient.get(DOWNLOAD_LOGS_ENDPOINT, {responseType: 'blob' as 'json'});
    return this.httpClient.get(DOWNLOAD_LOGS_ENDPOINT, {responseType: 'blob' as 'json', observe: 'response' as 'response'}).pipe(
      //tap(resp => console.log('request response', resp))
        catchError(err1 => this.handleError(err1)));
  }

  getSystemInfo(settingsList): Observable<any> {

    let values = '';
    let isFirst = true;

    settingsList.forEach(setting => {
      //uciList.push(settingsList[key]);
      if (isFirst)  {
        isFirst = false;
      } else {
        values += '&';
      }
      values += setting;
    });

    const url = SYS_INFO_ENDPOINT + '?' + values;
    //console.log('REST SYS get "%s"', url);
    return this.httpClient.get<any>(url, this.httpOptions).pipe(
        catchError(err1 => this.handleError(err1)));
  }


  getSystemType(): Observable<any> {
    //console.log('SYS_TYPE_ENDPOINT get "%s"', SYS_TYPE_ENDPOINT);
    return this.httpClient.get<any>(SYS_TYPE_ENDPOINT, this.httpOptions).pipe(
        catchError(err1 => this.handleError(err1)));
  }


  private handleError(error: HttpErrorResponse) {
    if (error.error instanceof ErrorEvent) {
      // A client-side or network error occurred. Handle it accordingly.
      console.error('An error occurred:', error.error.message);
    } else {
      // The backend returned an unsuccessful response code.
      // The response body may contain clues as to what went wrong.
      console.error(
        `Backend returned code ${error.status}, ` +
        `body was: ${error.error}`);
    }
    // Return an observable with a user-facing error message.
    //return throwError('Something bad happened; please try again later.');
    return of(error); //returns HttpErrorResponse with status 0
  }

}
