import { Injectable } from '@angular/core';
import {environment} from "../../environments/environment";
import {HttpClient, HttpErrorResponse} from "@angular/common/http";
import {catchError} from "rxjs/operators";
import {Observable, of} from "rxjs";

@Injectable({
  providedIn: 'root'
})
export class UsersManagementService {

    USERS_ENDPOINT = environment.usersEndpoint;

  constructor(private httpClient: HttpClient) { }

    getUsers(): Observable<any> {
        return this.httpClient.get(this.USERS_ENDPOINT, {observe: 'response' as 'response', responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    saveNewUser(userData): Observable<any> {
        return this.httpClient.post(this.USERS_ENDPOINT, userData, {
            headers: {'Content-Type': 'application/json'},
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    saveUserChanges(userData): Observable<any> {
        return this.httpClient.put(this.USERS_ENDPOINT, userData, {
            headers: {'Content-Type': 'application/json'},
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    deleteUsers(userIds): Observable<any> {
        return this.httpClient.delete(this.USERS_ENDPOINT, {
            body: userIds,
            headers: {'Content-Type': 'application/json'},
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    renewToken(userId): Observable<any> {
        return this.httpClient.post(this.USERS_ENDPOINT + "/" + userId + "/token",null, {
            observe: 'response' as 'response',
            responseType: 'text'})
            .pipe(
                catchError(err1 => this.handleError(err1)));
    }

    private handleError(error: HttpErrorResponse) {
        if (error.error instanceof ErrorEvent) {
            // A client-side or network error occurred. Handle it accordingly.
            console.error('An error occurred:', error.error.message);
        } else {
            // The backend returned an unsuccessful response code.
            // The response body may contain clues as to what went wrong.
            console.error(
                `Backend returned code ${error.status}, ` +
                `body was: ${error.error}`);
        }
        // Return an observable with a user-facing error message.
        //return throwError('Something bad happened; please try again later.');
        return of(error); //returns HttpErrorResponse with status 0
    }
}
