import { AbstractControl, FormControl, UntypedFormGroup, ValidationErrors, ValidatorFn, Validators } from '@angular/forms';

export const settingsPatternValidators = {
    GENERAL_IPV4_ADDRESS_REGEX : "(^$)|(^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$)",
    ALIAS_REGEX : "^[\\w\\s#|&\\/.,\\-]*$"
}

export function buildSettingsFormGroup(formGroup: UntypedFormGroup, uciData) {
    Object.keys(uciData).forEach( (uci) => {
        switch (uci) {
            case "device.device.alias":
                formGroup.addControl('alias', new FormControl(uciData["device.device.alias"], Validators.pattern(settingsPatternValidators.ALIAS_REGEX)));
                break;
            case "sonic_ip.general.enabled":
                formGroup.addControl('sonic_ip', new FormControl<boolean>(grantIsBoolean(uciData["sonic_ip.general.enabled"]), Validators.required));
                break;
            case "sonic_ip.general.volume":
                formGroup.addControl('sonic_ip_vol', new FormControl(uciData["sonic_ip.general.volume"], [Validators.min(0), Validators.max(100), Validators.required]));
                break;
            case "network.eth0.proto":
                formGroup.addControl('method', new FormControl(uciData["network.eth0.proto"], Validators.required));
                break;
            case "network.eth0.dhcpname":
                formGroup.addControl('dhcpname', new FormControl(uciData["network.eth0.dhcpname"]));
                break;
            case "network.eth0.ipaddr":
                formGroup.addControl('static_ip_addr', new FormControl(uciData["network.eth0.ipaddr"], Validators.pattern(settingsPatternValidators.GENERAL_IPV4_ADDRESS_REGEX)));
                break;
            case "network.eth0.netmask":
                formGroup.addControl('static_netmask', new FormControl(uciData["network.eth0.netmask"], Validators.pattern(settingsPatternValidators.GENERAL_IPV4_ADDRESS_REGEX)));
                break;
            case "network.eth0.gateway":
                formGroup.addControl('static_gateway', new FormControl(uciData["network.eth0.gateway"], Validators.pattern(settingsPatternValidators.GENERAL_IPV4_ADDRESS_REGEX)));
                break;
            case "network.eth0.dns_type":
                formGroup.addControl('dns_type', new FormControl(uciData["network.eth0.dns_type"], Validators.required));
                break;
            case "network.eth0.dns1":
                formGroup.addControl('dns_server1', new FormControl(uciData["network.eth0.dns1"], Validators.pattern(settingsPatternValidators.GENERAL_IPV4_ADDRESS_REGEX)));
                break;
            case "network.eth0.dns2":
                formGroup.addControl('dns_server2', new FormControl(uciData["network.eth0.dns2"], Validators.pattern(settingsPatternValidators.GENERAL_IPV4_ADDRESS_REGEX)));
                break;
            case "ntp.source.server1":
                formGroup.addControl('ntp_server1', new FormControl(uciData["ntp.source.server1"]));
                break;
            case "ntp.source.server2":
                formGroup.addControl('ntp_server2', new FormControl(uciData["ntp.source.server2"]));
                break;
            case "ntp.source.server3":
                formGroup.addControl('ntp_server3', new FormControl(uciData["ntp.source.server3"]));
                break;
            case "security.defaults.enabled":
                formGroup.addControl('defaults_opt', new FormControl<boolean>(grantIsBoolean(uciData["security.defaults.enabled"]), Validators.required));
                break;
            case "security.update.enabled":
                formGroup.addControl('update_opt', new FormControl<boolean>(grantIsBoolean(uciData["security.update.enabled"]), Validators.required));
                break;
            case "security.reset.enabled":
                formGroup.addControl('reboot_opt', new FormControl<boolean>(grantIsBoolean(uciData["security.reset.enabled"]), Validators.required));
                break;
            case "dropbear.RunCtl.enable":
                formGroup.addControl('ssh', new FormControl<boolean>(grantIsBoolean(uciData["dropbear.RunCtl.enable"]), Validators.required));
                break;
            case "httpd.webserver.session_timeout":
                formGroup.addControl('session_timeout', new FormControl(uciData["httpd.webserver.session_timeout"], Validators.pattern('^\\d*$')));
                break;
            default:
                // console.error("Unrecognized UCI", uci);
        }
    });


    function grantIsBoolean(value: any): boolean {
        return Boolean(JSON.parse("" + value));
    }
}
