import { Component, OnInit } from '@angular/core';
import {SystemInfoService} from "../services/system-info.service";
import {DeviceConfigService} from "../services/device-config.service";
import {Router} from '@angular/router';
import {HttpErrorResponse} from '@angular/common/http';
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {MatDialog, MatDialogConfig} from "@angular/material/dialog";
import {OpenSourceLicensesComponent} from "../open-source-licenses/open-source-licenses.component";
import {ToastrService} from "ngx-toastr";
import {ToastMessageService} from "../services/toast-message.service";
import {timezoneOffsetToString} from "../main/datetime_utils";
import { TopBarService } from '../services/top-bar.service';

@Component({
  selector: 'app-system-status',
  templateUrl: './system-status.component.html',
  styleUrls: ['./system-status.component.css', '../css/general.css']
})
export class SystemStatusComponent implements OnInit {

  loadFinished = false;

  // DevInfoPanelOpenState = false;
  // NetworkPanelOpenState = false;
  // SystemPanelOpenState = false;
  // LicensesPanelOpenState = false;
  // StorageDevPanelOpenState = false;
  // PoEPanelOpenState = false;
  // AmplifierPanelOpenState = false;

  device_type = "";
  device_type_ID = "";
  IPAM_type = "";
  IPAM_type_ID = "";
  kernel_ver = "";
  boot_ver = "";

  connection_status = "";
  mac_addr = "";
  ip_addr = "";
  netmask = "";
  default_gateway = "";
  DNS_servers = [];
  fw_name = "";
  fw_ver = "";
  app = "";
  app_ver = "";


  local_time : number;
  timezoneOffset : string;
  uptimeSec: number;
  uptimeStr = "";

  licenses = [];

  massStorageDevs = [];
  massStorageDevsTableColums = ["type", "usage", "size", "used", "available", "fsType", "mountAs"];

  openSourceLicenses = [];

  HW_type_ID : string;
  hasPoEStatus = false;
  hasTempSensor = false;
  poe_power_source = "";
  poe_speaker_impedance = "";
  poe_amp_gain = "";
  otw_status = 0;
  amp_temp : string;
  sensor_temp : string;

  timeTimer: NodeJS.Timeout | null = null;

  //settingsList = ["device_type" , "device_type_ID", "IPAM_type", "IPAM_type_ID", "mac_addr", "conn_status", "ip_addr", "netmask", "default_gateway", "DSN_servers", "kernelVer", "bootloaderVer", "sys_time", "uptime", "licenses", "massStorageDev", "otw_status", "open_source_licenses"];
  settingsList = ["device_type" , "device_type_ID", "IPAM_type", "IPAM_type_ID", "mac_addr", "conn_status", "ip_addr", "netmask", "default_gateway", "DSN_servers", "kernelVer", "bootloaderVer", "local_time", "uptime", "licenses", "massStorageDev", "open_source_licenses"];


  constructor(private systemInfoService: SystemInfoService,
              private configsService: DeviceConfigService,
              private router: Router,
              private httpResponseHandler: HTTPResponseHandlerService,
              public matDialog: MatDialog,
              private toast: ToastMessageService,
              private topBarService: TopBarService) { }

  ngOnInit() {
    this.toast.clearToastMessages()
    this.systemInfoService.getSystemType().subscribe(data => {
      //console.log("Status:", data.status);
      //console.log("Received:", data.body);
      switch (data.status) {
        case 200:

          if (data.body.hasOwnProperty('hasPoESettings')){
            this.hasPoEStatus = true;
            this.settingsList = this.settingsList.concat(['poe_power_source', 'poe_speaker_impedance', 'poe_amp_gain']);
          }

          if (data.body.hasOwnProperty('HW_type_ID')){
            this.HW_type_ID = data.body['HW_type_ID'];
            if (data.body['HW_type_ID'] === '66'){ //TPA400
              this.settingsList.push('amp_temp');
              //console.log(this.settingsList);
            }
          }

          if (data.body.hasOwnProperty('HW_type_ID')){
            this.HW_type_ID = data.body['HW_type_ID'];
            if (data.body['HW_type_ID'] === '69'){ //MPA400
              this.settingsList.push('otw_status');
              //console.log(this.settingsList);
            }
          }

          if (data.body.hasOwnProperty('hasTempSensor')){
            this.hasTempSensor = true;
            this.settingsList = this.settingsList.concat(['sensor_temp']);
          }

          //console.log(this.settingsList);
          this.systemInfoService.getSystemInfo(this.settingsList).subscribe(data => {
            //console.log("Status:", data.status);
            //console.log("Received:", data.body);
            switch (data.status) {
              case 200:
                this.device_type = data.body["device_type"];
                this.device_type_ID = data.body["device_type_ID"];
                this.IPAM_type = data.body["IPAM_type"];
                this.IPAM_type_ID = "ID "+ data.body["IPAM_type_ID"];
                this.mac_addr = data.body["mac_addr"];
                this.kernel_ver = data.body["kernelVer"];
                this.boot_ver = data.body["bootloaderVer"];

                if (data.body["conn_status"] === "up"){
                  this.connection_status = "Connected";
                } else {
                  this.connection_status = "Disconnected";
                }

                this.ip_addr = data.body["ip_addr"];
                this.netmask = data.body["netmask"];
                this.default_gateway = data.body["default_gateway"];
                if (typeof (data.body["DSN_servers"]) === "string"){
                  this.DNS_servers.push({'value' : data.body["DSN_servers"], 'style' : 'normal'});
                } else {
                  let dns_servers_list = data.body["DSN_servers"];
                  if (dns_servers_list.length === 2){
                    this.DNS_servers.push({'value' : dns_servers_list[0], 'style' : 'normal'});
                    this.DNS_servers.push({'value' : dns_servers_list[1], 'style' : 'normal'});
                  } else if (dns_servers_list.length === 1) {
                    this.DNS_servers.push({'value' : dns_servers_list[0], 'style' : 'normal'});
                    //this.DNS_servers.push({'value' : "Not configured", 'style' : 'italic'});
                  } else {
                    this.DNS_servers.push({'value' : "Not configured", 'style' : 'italic'});
                    //this.DNS_servers.push({'value' : "Not configured", 'style' : 'italic'});
                  }
                }


                /*this.fw_name = data.body["fw_name"];
                this.fw_ver = data.body["fw_version"];
                this.app = data.body["app"];
                this.app_ver = data.body["app_ver"];*/

                //this.sys_time = (data.body["sys_time"]*1000);
                // let sys_dateTime = new Date(data.body["sys_time"]*1000);
                // let daysOfWeek = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
                // let monthsOfYear = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
                // this.sys_time = daysOfWeek[sys_dateTime.getUTCDay()] + " " + monthsOfYear[sys_dateTime.getUTCMonth()] + " " + sys_dateTime.getUTCDate() + " " + sys_dateTime.getUTCHours().toString().padStart(2, '0') + ":" + sys_dateTime.getUTCMinutes().toString().padStart(2, '0') + ":" + sys_dateTime.getUTCSeconds().toString().padStart(2, '0') + " UTC " + sys_dateTime.getUTCFullYear()
                this.local_time = data.body["local_time"][0] * 1000;
                this.timezoneOffset =  timezoneOffsetToString(data.body["local_time"][1]);
                this.uptimeSec = data.body["uptime"];

                this.timeTimer = setInterval(() => {
                  this.local_time += 1000;
                  this.uptimeStr = this.calculateTimeDiff(this.uptimeSec, data.body["local_time"][0]);
                  this.uptimeSec -= 1;
                }, 1000);

                if (data.body['licenses'] !== null){
                  this.licenses = data.body['licenses']
                }

                this.massStorageDevs = data.body["massStorageDev"];

                this.otw_status = data.body['otw_status'];

                this.openSourceLicenses = data.body["open_source_licenses"];

                if (this.hasPoEStatus){
                  this.poe_power_source = data.body['poe_power_source'];
                  this.poe_speaker_impedance = data.body['poe_speaker_impedance'];
                  this.poe_amp_gain = data.body['poe_amp_gain'];
                }

                if (this.HW_type_ID == '66'){
                  this.amp_temp = data.body['amp_temp'];
                }

                if (this.hasTempSensor) {
                  this.sensor_temp = data.body['sensor_temp'].replace('\n', '');
                }

                this.loadFinished = true;

                break;
              case 401:
                this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
                break;
              case 500: //INTERNAL_SERVER_ERROR
                this.httpResponseHandler.handle500HttpErrorResponse();
                this.loadFinished = true;
                break;
              default:
                console.log("Unexpected return code: ", data.status);
                //this.httpResponseHandler.handleUnexpectedHttpErrorResponse(data.status);
                this.loadFinished = true;
                break;
            }
          });

          break;
        case 401:
          this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
          break;
        case 500:
          this.httpResponseHandler.handle500HttpErrorResponse();
          break;
        default:
          console.log("Unexpected return code: ", data.status);
          //this.httpResponseHandler.handleUnexpectedHttpErrorResponse(data.status);
          break;
      }
    });
    this.topBarService.updateDeviceLocalTime();
  }

  /*convertPyTimestampToDate(pyTimestamp){
    let timestamp = parseInt(String(pyTimestamp*1000), 10); //convert ms to sec and float to integer
    //let res = parseInt(String(timestamp), 10);
    //console.log(res);
    return new Date(timestamp).toString();
  }*/

  calculateTimeDiff(pyTimestamp, sys_dateTime) {
    let timediff = Math.round(sys_dateTime) - pyTimestamp; //javascript timestamp is in seconds
    let tMins = Math.floor(timediff / 60);
    let nSecs = (timediff % 60);
    let tHours = Math.floor(tMins / 60);
    let nMins = (tMins % 60);
    let nDays = Math.floor(tHours / 24);
    let nHours = (tHours % 24);
    return nDays + " days, " + nHours + " hours, " + nMins + " minutes, " + nSecs + " seconds";
  }


  showOpenSourceLibsLicenses() {
    const dialogConfig = new MatDialogConfig();
    // The user can't close the dialog by clicking outside its body
    //dialogConfig.disableClose = false;
    dialogConfig.id = "warning-popup-modalC";
    dialogConfig.height = "90%";
    dialogConfig.width = "1000px";
    dialogConfig.data = this.openSourceLicenses;
    this.matDialog.open(OpenSourceLicensesComponent, dialogConfig);
  }


  ngOnDestroy(){
    if (this.timeTimer) {
      clearInterval(this.timeTimer);
    }
  }

}
