import {
    animate, group,
    keyframes,
    state,
    style,
    transition,
    trigger
} from '@angular/animations';
import {Component} from '@angular/core';
import {Toast, ToastrService, ToastPackage} from 'ngx-toastr';


@Component({
    selector: '[app-toast-message]',
    templateUrl: './toast-message.component.html',
    styleUrls: ['./toast-message.component.css'],
    animations: [
        trigger('flyInOut', [
            state('inactive', style({
                opacity: 0,
            })),
            transition('inactive => active', animate('500ms ease-out', keyframes([
                style({
                    transform: 'translate3d(100%, 0, 0)',
                    opacity: 0,
                }),
                style({
                    transform: 'none',
                    opacity: 1,
                }),
            ]))),
            transition('active => removed', animate('400ms ease-out', keyframes([
                style({
                    opacity: 1,
                }),
                style({
                    transform: 'translate3d(100%, 0, 0) skewX(30deg)',
                    opacity: 0,
                }),
            ]))),
        ]),
    ],
    preserveWhitespaces: false,
})


export class ToastMessageComponent extends Toast {

    undoString = 'undo';

    constructor(protected toastrService: ToastrService,
                public toastPackage: ToastPackage) {

        super(toastrService, toastPackage);
    }

    action(event: Event) {
        event.stopPropagation();
        this.undoString = 'undid';
        this.toastPackage.triggerAction();
        return false;
    }

}
