import {Component, OnInit} from '@angular/core';
import {DeviceConfigService} from "../services/device-config.service";
import {SystemInfoService} from "../services/system-info.service";
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {HttpErrorResponse} from '@angular/common/http';
import {ToastrService} from "ngx-toastr";
import {ToastMessageService} from "../services/toast-message.service";
import {FormCanDeactivate} from "../can-deactivate/form-can-deactivate";
import {
    AbstractControl,
    FormControl,
    FormGroup,
    UntypedFormGroup,
    ValidationErrors,
    ValidatorFn,
    Validators
} from '@angular/forms';
import { TopBarService } from '../services/top-bar.service';


@Component({
    selector: 'app-update',
    templateUrl: './update-tab.component.html',
    styleUrls: ['../css/general.css', '../css/mat-form-field.css', './update-tab.component.css']
})


export class UpdateTabComponent implements OnInit {

    loadFinished = false;
    updating = false;
    update_opt = false;
    fw_version = "";
    rootFSDate = "";
    webUIVer = "";
    kernelVer = "";
    bootloaderVer = "";
    href = "";

    fileExtensionsAllowed = ['.tar'];
    max_file_size = 256 * 1024 * 1024;

    ucisList = {
        'update_opt': 'security.update.enabled',
        'fw_update_auto': 'application.fw_update.auto',
        'fw_update_url': 'application.fw_update.url',
        'fw_update_start_time': 'application.fw_update.start_time',
        'fw_update_stop_time': 'application.fw_update.stop_time',
        'fw_update_weekdays': 'application.fw_update.week_days'
    }

    currentUpdateStatus : string;

    constructor(private getSystemInfo: SystemInfoService,
                private configsService: DeviceConfigService,
                private httpResponseHandler: HTTPResponseHandlerService,
                private toast: ToastMessageService,
                private topBarService: TopBarService) {}

    // canDeactivateComponent() {
    //     console.log("current status", this.currentUpdateStatus);
    //     if (this.currentUpdateStatus === "uploading"){
    //         if (!confirm("If you leave, the firmware upload will be canceled!")){
    //             return false;
    //         }
    //     }
    //     // else if (this.currentUpdateStatus === "installing") {
    //     //     if (!confirm("If you leave, the firmware upload will be canceled!")){
    //     //         return false;
    //     //     }
    //     // }
    //     // if (this.isUpdating) {
    //     //     if (!confirm("ATTENTION! If you leave, the update will be interrupted, and your device might get damaged!")){
    //     //         return false;
    //     //     }
    //     // }
    //     // return true;
    //     // return !this.isUpdating;
    //     return true;
    // };

    ngOnInit() {
        this.toast.clearToastMessages()
        const sysInfoList = ["fw_version", "rootFSDate", "webUIVer", "kernelVer", "bootloaderVer"];
        this.getSystemInfo.getSystemInfo(sysInfoList).subscribe(data => {
            switch (data.status) {
                case 200:
                    this.fw_version = data.body["fw_version"];
                    this.rootFSDate = data.body["rootFSDate"];
                    this.webUIVer = data.body["webUIVer"];
                    this.kernelVer = data.body["kernelVer"];
                    this.bootloaderVer = data.body["bootloaderVer"];
                    break;

                case 401:
                    this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
                    break;

                case 500:
                    this.httpResponseHandler.handle500HttpErrorResponse();
                    break;

                default:
                    console.error("Unexpected return code: ", data.status);
            }

            let url_parts = window.location.href.split('/');
            let ip = url_parts[2].split(':')[0];
            this.href = url_parts[0] + "//" + ip + "/sys/uifloader.html";
            this.configsService.getUcis(this.ucisList).subscribe(data => {
                switch (data.status) {
                    case 200:
                        //build device form group
                        if (data.body["security.update.enabled"] === "true") {
                            this.update_opt = true;
                        }
                        this.loadFinished = true;
                        break;

                    case 401:
                        this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
                        break;

                    case 500:
                        this.httpResponseHandler.handle500HttpErrorResponse();
                        break;

                    default:
                        console.error("Unexpected return code: ", data.status);
                }

            });
        });
        this.topBarService.updateDeviceLocalTime();
    }

    // buildForm(ucis) {
    //     Object.keys(ucis).forEach( (uci) => {
    //         switch (uci) {
    //             case "application.fw_update.auto":
    //                 this.automaticUpdatesForm.addControl('fw_update_auto', new FormControl<boolean>(Boolean(JSON.parse("" + ucis["application.fw_update.auto"])), Validators.required));
    //                 break;
    //             case "application.fw_update.url":
    //                 this.automaticUpdatesForm.addControl('fw_update_url', new FormControl<string>(ucis["application.fw_update.url"], Validators.required));
    //                 break;
    //             case "application.fw_update.start_time":
    //                 this.automaticUpdatesForm.addControl('fw_update_start_time', new FormControl<string>(ucis["application.fw_update.start_time"], this.timeValidator()));
    //                 break;
    //             case "application.fw_update.stop_time":
    //                 this.automaticUpdatesForm.addControl('fw_update_stop_time', new FormControl<string>(ucis["application.fw_update.stop_time"], this.timeValidator()));
    //                 break;
    //             case "application.fw_update.week_days":
    //                 this.automaticUpdatesForm.addControl('fw_update_weekdays', new FormControl<string[]>(ucis["application.fw_update.week_days"] !== '' ? ucis["application.fw_update.week_days"].split(",") : []));
    //                 break;
    //             default:
    //         }
    //     });
    // }

    // timeValidator(): ValidatorFn {
    //     return (control: AbstractControl): ValidationErrors | null => {
    //         const timeRegex = new RegExp('^((0?[0-9])|(1[0-9])|(2[0-3])):[0-5]?[0-9]$');
    //         const valid = timeRegex.test(control.value);
    //         return !valid ? {invalidDate: {value: control.value}} : null;
    //     };
    // }

    // submit(){
    //     let canSubmit = false;
    //     if (this.automaticUpdatesForm.valid) {
    //         canSubmit = true;
    //     } else {
    //         if (this.automaticUpdatesForm.get('fw_update_auto').valid) {
    //             if ((this.automaticUpdatesForm.get('fw_update_url').invalid || this.automaticUpdatesForm.get('fw_update_start_time').invalid || this.automaticUpdatesForm.get('fw_update_stop_time').invalid || this.automaticUpdatesForm.get('fw_update_weekdays').invalid) && !this.automaticUpdatesForm.get('fw_update_auto').value) {
    //                 canSubmit = true;
    //             }
    //         }
    //     }
    //     if (canSubmit) {
    //         this.show_settings_component = false;
    //         this.show_loading_component = true; // start loading spinner
    //         setTimeout(() => { //wait a bit to be able to see spinner for usability
    //             //build dict with ucis to send
    //             let settingsToSubmit = {};
    //             Object.keys(this.automaticUpdatesForm.value).forEach(key => {
    //                 if ((key === 'fw_update_url' || key === "fw_update_start_time" || key === 'fw_update_stop_time' || key === 'fw_update_weekdays') && !this.automaticUpdatesForm.get('fw_update_auto').value) {
    //                     //don't add
    //                 } else {
    //                     settingsToSubmit[this.ucisList[key]] = "" + this.automaticUpdatesForm.get(key).value;
    //                 }
    //             });
    //
    //             const payload = {
    //                 commit: true
    //             }
    //
    //             if (Object.keys(settingsToSubmit).length !== 0) {
    //                 payload['ucis'] = settingsToSubmit;
    //             }
    //             this.configsService.setSettings(payload).subscribe(data => {
    //                 switch (data.status) {
    //                     case 200: //SUCCESS
    //                         this.toast.showSuccessMessage("New configuration applied successfully.");
    //                         setTimeout(() => {
    //                             window.location.reload();
    //                         }, 2000);
    //                         break;
    //
    //                     case 400: //BAD_REQUEST
    //                         this.toast.showErrorMessage(data.error);
    //                         window.location.reload();
    //                         break;
    //
    //                     case 401:
    //                         this.httpResponseHandler.handle401HttpErrorResponse((data as HttpErrorResponse).error);
    //                         break;
    //
    //                     case 500: //INTERNAL_SERVER_ERROR
    //                         this.httpResponseHandler.handle500HttpErrorResponse();
    //                         break;
    //
    //                     default:
    //                         console.error("Unexpected return code: ", data.status);
    //                 }
    //             });
    //         }, 1300);
    //     } else {
    //         this.toast.showErrorMessage("Automatic updates configuration invalid!");
    //     }
    // }

    // cancelChanges() {
    //     this.automaticUpdatesForm.reset();
    // }

    updateStatus(status: string) {
        this.currentUpdateStatus = status;
    }

    // isSelected(weekday: string) : boolean {
    //     return (this.automaticUpdatesForm.get('fw_update_weekdays').value as string[]).indexOf(weekday) > -1;
    // }
    //
    // selectOrDeselectBtn(weekday: string) {
    //     const i = (this.automaticUpdatesForm.get("fw_update_weekdays").value as string[]).indexOf(weekday);
    //     if (i > -1) {
    //         let weekdayArray = this.automaticUpdatesForm.get("fw_update_weekdays").value as string[];
    //         weekdayArray.splice(i, 1);
    //         this.automaticUpdatesForm.get("fw_update_weekdays").patchValue(weekdayArray as never);
    //         this.automaticUpdatesForm.markAsDirty();
    //     } else {
    //         let weekdaysArray = this.automaticUpdatesForm.get("fw_update_weekdays").value as string[];
    //         weekdaysArray.push(weekday);
    //         this.automaticUpdatesForm.get("fw_update_weekdays").patchValue(weekdaysArray as never);
    //         this.automaticUpdatesForm.markAsDirty();
    //     }
    // }

    scrollToId(el: HTMLElement) {
        el.scrollIntoView({'behavior': 'smooth'});
    }

    getLoadingText() {
        return "Processing..."
    }

}
