import {Component, OnInit, EventEmitter, Output, Inject} from '@angular/core';
import { AdditionalCertificatesService } from '../services/additional.certificates.service';
import {MAT_DIALOG_DATA, MatDialogRef} from '@angular/material/dialog';
import {ToastrService} from 'ngx-toastr';
import {HTTPResponseHandlerService} from '../services/httpresponse-handler.service';
import {HttpErrorResponse, HttpEventType, HttpResponse} from '@angular/common/http';
import {ToastMessageService} from "../services/toast-message.service";
import {FormCanDeactivate} from "../can-deactivate/form-can-deactivate";

@Component({
  selector: 'app-upload-certificate',
  templateUrl: './upload-files-modal.component.html',
  styleUrls: ['./upload-files-modal.component.css', '../css/general.css']
})
export class UploadFilesModalComponent implements OnInit {

    constructor(private fileUploadService: AdditionalCertificatesService,
                public dialogRef: MatDialogRef<UploadFilesModalComponent>,
                private toastr: ToastrService,
                @Inject(MAT_DIALOG_DATA) public data) {
    }

    filesList = []; //variable to store file
    invalid : boolean;

    ngOnInit(): void {
        this.invalid = undefined;
    }

    onChange(event){
        // check if file is valid
        this.filesList = event.target.files;
        this.invalid = false;
        if (this.filesList.length === 0){
            this.invalid = true;
        } else {
            for (let file of this.filesList){
                //validate file extension
                let fileExtension = file.name.split('.').pop();
                if (this.data["FILE_EXTENSIONS"] && !this.data["FILE_EXTENSIONS"].includes(fileExtension.toLowerCase())){
                    this.invalid = true;
                    //this.toast.showWarningMessage("\""+file.name+"\": format not supported!");
                    this.toastr.show("\""+file.name+"\": format not supported!", '', {closeButton : true, timeOut: 10000, toastClass: 'my-toast-warning', ...(this.data["toastMsgPositionClass"] !== '') && {positionClass: this.data["toastMsgPositionClass"]}});
                    break;
                } else {
                    //validate file size
                    if (file.size > this.data["MAX_FILE_SIZE"]) {
                        this.invalid = true;
                        //this.toast.showWarningMessage("\""+file.name+"\": file size is too large!");
                        this.toastr.show("\""+file.name+"\": file size is too large!", '', {closeButton : true, timeOut: 10000, toastClass: 'my-toast-warning', ...(this.data["toastMsgPositionClass"] !== '') && {positionClass: this.data["toastMsgPositionClass"]}});
                        break;
                    } else {
                        //console.log(file.size);
                        if (this.data["AVAILABLE_STORAGE_SPACE"] !== undefined && file.size > this.data["AVAILABLE_STORAGE_SPACE"]){
                            this.invalid = true;
                            //this.toast.showWarningMessage("This file upload exceeds device's free storage space!");
                            this.toastr.show("This file upload exceeds device's free storage space!", '', {closeButton : true, timeOut: 10000, toastClass: 'my-toast-warning', ...(this.data["toastMsgPositionClass"] !== '') && {positionClass: this.data["toastMsgPositionClass"]}});
                        } else {
                            if (Object.keys(this.data).includes("FILE_NAME_VALIDATION")) {
                                //validate filename
                                let fileName = file.name.replace('.'+fileExtension, '');
                                const regex = RegExp(this.data["FILE_NAME_VALIDATION"]);
                                if (!regex.test(fileName)){
                                    this.invalid = true;
                                    //this.toast.showWarningMessage("Only filenames with alphanumeric characters and one space in between allowed!");
                                    this.toastr.show("Only filenames with alphanumeric characters and one space in between allowed!", '', {closeButton : true, timeOut: 10000, toastClass: 'my-toast-warning', ...(this.data["toastMsgPositionClass"] !== '') && {positionClass: this.data["toastMsgPositionClass"]}});
                                }
                            }
                        }
                    }
                }
            }
        }
        if (!this.invalid && this.data["AVAILABLE_STORAGE_SPACE"] !== undefined){
            let totalSize = 0;
            for (let file of this.filesList){
                totalSize += file.size;
            }
            if (totalSize > this.data["AVAILABLE_STORAGE_SPACE"]) {
                this.invalid = true;
                //this.toast.showWarningMessage("This file upload exceeds device's free storage space!");
                this.toastr.show("This file upload exceeds device's free storage space!", '', {closeButton : true, timeOut: 10000, toastClass: 'my-toast-warning', ...(this.data["toastMsgPositionClass"] !== '') && {positionClass: this.data["toastMsgPositionClass"]}});
            }
        }
    }

    onUpload() {
        if (!this.invalid) {
            this.dialogRef.close(this.filesList);
        }
    }

    closeModal() {
        this.dialogRef.close();
    }

}
